--------------------------------------------------------------------------------
-- CERN (BE-CO-HT)
-- Top level entity of CONV-TTL-BLO V2
-- http://www.ohwr.org/projects/conv-ttl-blo
--------------------------------------------------------------------------------
--
-- unit name: image1_core.vhd
--
-- author: Carlos Gil Soriano (gilsoriano@gmail.com)
--         Theodor-Adrian Stana (t.stana@cern.ch)
--
-- date: 01-12-2012
--
-- version: 0.9
--
-- description: Core of image1 for CONV-TTL-BLO V2
--
-- dependencies:
--
-- references:
--    [1] ELMA Crates Specification: Access to board data using SNMP and I2C
--              available at: http://www.ohwr.org/documents/227
--
--------------------------------------------------------------------------------
-- GNU LESSER GENERAL PUBLIC LICENSE
--------------------------------------------------------------------------------
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--------------------------------------------------------------------------------


library IEEE;
library unisim;
library work;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.image1_pkg.ALL;
use work.image1_wrappers_pkg.ALL;
use work.image1_led_pkg.ALL;
use work.wishbone_pkg.ALL;
use work.bicolor_led_ctrl_pkg.ALL;
use UNISIM.VCOMPONENTS.ALL;

entity image1_core is
  generic
  (
    g_NUMBER_OF_CHANNELS : NATURAL := 6
  );
  port 
  (
    rst_i             : in  std_logic;
    clk_20_i          : in  std_logic;
    clk_125_i         : in  std_logic;
    
    -- LEDs
    led_array_o          : out t_led_array_o;
    led_front_n          : out std_logic_vector(g_NUMBER_OF_CHANNELS downto 1);
    led_rear_n           : out std_logic_vector(g_NUMBER_OF_CHANNELS downto 1);
    
    -- I/Os for pulses
    pulse_i_front_n      : in  std_logic_vector(g_NUMBER_OF_CHANNELS downto 1);
    pulse_o_front        : out std_logic_vector(g_NUMBER_OF_CHANNELS downto 1);
    pulse_i_rear         : in  std_logic_vector(g_NUMBER_OF_CHANNELS downto 1);
    pulse_o_rear         : out std_logic_vector(g_NUMBER_OF_CHANNELS downto 1);
    inv_i_n              : in  std_logic_vector(4 downto 1);
    inv_o                : out std_logic_vector(4 downto 1);
    
    -- Lines for the i2c_slave
    i2c_slave_i          : in  t_i2c_slave_i;
    i2c_slave_o          : out t_i2c_slave_o;
    
    -- FPGA Geographical address pins (reused for i2c address)
    FPGA_GA              : in  std_logic_vector(4 downto 0);
    FPGA_GAP             : in  std_logic;
    
    -- Pins of the SPI interface to write into the Flash memory
    spi_master_i         : in  t_spi_master_i;
    spi_master_o         : out t_spi_master_o;
    

    -- Enable signals
    fpga_o_en            : out std_logic;
    fpga_o_blo_en        : out std_logic;
    fpga_o_ttl_en        : out std_logic;
    fpga_o_inv_en        : out std_logic;

    -- Level and switch inputs
    level_i              : in  std_logic;
    switch_i             : in  std_logic;
    
    -- Manual reset, allows power sequencing of the
    -- 24V rail after a security given delay
    manual_rst_n_o       : out std_logic;
    
    -- RTM identifiers
    rtmm_i               : in  std_logic_vector(2 downto 0);
    rtmp_i               : in  std_logic_vector(2 downto 0)
  );
end image1_core;


architecture Behavioral of image1_core is

  --============================================================================
  -- Component declarations
  --============================================================================
  -- Trigger leds I2C test component
  component test_trigleds_wb is
    port 
    (
      rst_n_i                                  : in     std_logic;
      clk_sys_i                                : in     std_logic;
      wb_dat_i                                 : in     std_logic_vector(31 downto 0);
      wb_dat_o                                 : out    std_logic_vector(31 downto 0);
      wb_cyc_i                                 : in     std_logic;
      wb_sel_i                                 : in     std_logic_vector(3 downto 0);
      wb_stb_i                                 : in     std_logic;
      wb_we_i                                  : in     std_logic;
      wb_ack_o                                 : out    std_logic;
      wb_stall_o                               : out    std_logic;
      -- Port for std_logic_vector field: 'Bits' in reg: 'LED'
      trigleds_reg_bits_o                      : out    std_logic_vector(5 downto 0)
    );
  end component test_trigleds_wb;      


  -- I2C bridge top-level
  component i2c_slave_top
    generic
    (
      g_WB_CLK_PERIOD : TIME := c_WB_CLK_PERIOD  -- Specify in ns
    ); 
    port
    (
      sda_oen          : out std_logic;
      sda_i            : in  std_logic;
      sda_o            : out std_logic;
      scl_oen          : out std_logic;
      scl_i            : in  std_logic;
      scl_o            : out std_logic;

      wb_clk_i         : in  std_logic;
      wb_rst_i         : in  std_logic;

      wb_master_stb_o  : out std_logic;
      wb_master_cyc_o  : out std_logic;
      wb_master_sel_o  : out std_logic_vector(3  downto 0);
      wb_master_we_o   : out std_logic;
      wb_master_data_i : in  std_logic_vector(31 downto 0);
      wb_master_data_o : out std_logic_vector(31 downto 0);
      wb_master_addr_o : out std_logic_vector(15  downto 0);
      wb_master_ack_i  : in  std_logic;
      wb_master_rty_i  : in  std_logic;
      wb_master_err_i  : in  std_logic;

      wb_slave_stb_i   : in  std_logic;
      wb_slave_cyc_i   : in  std_logic;
      wb_slave_sel_i   : in  std_logic_vector(3  downto 0);
      wb_slave_we_i    : in  std_logic;
      wb_slave_data_i  : in  std_logic_vector(31 downto 0);
      wb_slave_data_o  : out std_logic_vector(31 downto 0);
      wb_slave_addr_i  : in  std_logic_vector(3  downto 0);
      wb_slave_ack_o   : out std_logic;
      wb_slave_rty_o   : out std_logic;
      wb_slave_err_o   : out std_logic;

      pf_wb_addr_o     : out std_logic;
      rd_done_o        : out std_logic;
      wr_done_o        : out std_logic;
      i2c_addr_i       : in  std_logic_vector(6  downto 0)
    );
  end component i2c_slave_top;
  
  
  -- RTM detector component
  component rtm_detector is
  port 
  (
    rtmm_i    : in std_logic_vector(2 downto 0);
    rtmp_i    : in std_logic_vector(2 downto 0);
    rtmm_ok_o : out std_logic;
    rtmp_ok_o : out std_logic
  );
  end component rtm_detector;

  --============================================================================
  -- Signal declarations
  --============================================================================
  -- Clock and reset signals
  signal clk_buf_50  : std_logic;
  signal clk_50      : std_logic;
  signal clk_buf_200 : std_logic;
  signal clk_200     : std_logic;
  signal pll_fb_out  : std_logic;
  signal pll_fb_in   : std_logic;
  signal rst_n       : std_logic;
  
  -- Signal to indicate the lock status of the PLL
  signal pll_locked  : std_logic;

  -- RTM detection signals
  signal rtmm_ok  : std_logic;
  signal rtmp_ok  : std_logic;

  -- Wishbone crossbar signals
  signal xbar_slave_in  : t_wishbone_slave_in_array   (c_NUM_MASTERS - 1 downto 0);
  signal xbar_slave_out  : t_wishbone_slave_out_array  (c_NUM_MASTERS - 1 downto 0);
  signal xbar_master_in : t_wishbone_master_in_array  (c_NUM_SLAVES  - 1 downto 0);
  signal xbar_master_out : t_wishbone_master_out_array (c_NUM_SLAVES  - 1 downto 0);

  -- Internal I2C signals
  signal i2c_addr    : std_logic_vector(6 downto 0);
  signal i2c_rd_done : std_logic;
  signal i2c_wr_done : std_logic;
  signal i2c_up      : std_logic;

  -- Internal pulse signal
  signal pulse_front_in     : std_logic_vector(g_NUMBER_OF_CHANNELS downto 1);
  
  -- Internal signals for bicolor LED matrix control
  signal leds_array : t_leds_array;
  signal led_state_array   : t_led_state_array(c_NB_ARRAY_LEDS - 1 downto 0);

  -- !!!!!!!!!!!!!!!!!!!
  -- I2C test front panel LED signal
  signal trigleds : std_logic_vector(5 downto 0);

begin

  --============================================================================
  -- General I/O logic
  --============================================================================
  -- Assign the active low reset signal
  rst_n          <= not(rst_i);
  
  -- Negate input pulses, due to the Schmitt trigger inverter on the board
  pulse_front_in <= not(pulse_i_front_n);
  
  -- Assign the I2C address based on the geographical address lines (see [1])
  i2c_addr       <= "10" & FPGA_GA;

  --============================================================================
  -- Generation of internal clock signals
  --============================================================================
  -- set up the fabric PLL_BASE to drive the BUFPLL
  pll_base_inst : PLL_BASE
  generic map 
  (
    BANDWIDTH          => "OPTIMIZED",
    CLK_FEEDBACK       => "CLKFBOUT",
    COMPENSATION       => "SYSTEM_SYNCHRONOUS",
    DIVCLK_DIVIDE      => 1,  
    CLKFBOUT_MULT      => 8,       -- 1 GHz at FBOUT
    CLKFBOUT_PHASE     => 0.000,
    CLKOUT0_DIVIDE     => 20,      -- 50 MHz at OUT0
    CLKOUT0_PHASE      => 0.000,
    CLKOUT0_DUTY_CYCLE => 0.500,
    CLKOUT1_DIVIDE     => 5,       -- 200 MHz at OUT1
    CLKOUT1_PHASE      => 0.000,
    CLKOUT1_DUTY_CYCLE => 0.500,
    CLKIN_PERIOD       => c_CLKIN_PERIOD,
    REF_JITTER         => 0.010
  )
  port map 
  (
    -- Output clocks
    CLKFBOUT => pll_fb_out,
    CLKOUT0  => clk_buf_50,
    CLKOUT1  => clk_buf_200,
    CLKOUT2  => open,
    CLKOUT3  => open,
    CLKOUT4  => open,
    CLKOUT5  => open,
    -- Status and control signals
    LOCKED   => pll_locked,  
    RST      => rst_i, -- s_rst.PLL(c_RST_PLL_CLKS - 1),
    -- Input clock control
    CLKFBIN  => pll_fb_in,
    CLKIN    => clk_125_i
  );

  -- Clock buffer on the PLL feedback line
  s_clk_fb_bufg : BUFG
  port map
  (
    I => pll_fb_out,
    O => pll_fb_in
  );

  -- Clock buffer for distributing the 50 MHz clock signal
  s_clk_50M_bufg : BUFG
  port map 
  (
    I => clk_buf_50,
    O => clk_50
  );

  -- Clock buffer for distributing the 200 MHz clock signal
  s_clk_200M_bufg : BUFG
  port map 
  (
    I => clk_buf_200,
    O => clk_200
  );

  --============================================================================
  -- Pulse generation module
  --============================================================================
  cmp_basic_trigger: basic_trigger_top
  generic map
  (
    g_NUMBER_OF_CHANNELS   => c_NUMBER_OF_CHANNELS,
    g_CLK_PERIOD           => c_CLKB_PERIOD,
    g_OUTPUT_PULSE_LENGTH  => c_OUTPUT_PULSE_LENGTH,
    g_LED_BLINKING_LENGTH  => c_LED_BLINKING_LENGTH
  )
  port map
  (
    clk_i          => clk_200,
    rst_i          => rst_i, -- s_rst.SYS_B(c_RST_B_CLKS - 1),
    led_ttl_o      => leds_array.top.TTL_N,
    fpga_o_en      => fpga_o_en,
    fpga_o_ttl_en  => fpga_o_ttl_en,
    fpga_o_inv_en  => fpga_o_inv_en,
    fpga_o_blo_en  => fpga_o_blo_en,
    level_i        => level_i,
    switch_i       => switch_i,
    manual_rst_n_o => manual_rst_n_o,
    pulse_i_front  => pulse_front_in,
    pulse_o_front  => pulse_o_front,
    pulse_i_rear   => pulse_i_rear,
    pulse_o_rear   => pulse_o_rear,
    led_o_front    => open, -- led_front_n,
    led_o_rear     => led_rear_n,
    inv_i          => inv_i_n,
    inv_o          => inv_o
  );


  --============================================================================
  -- I2C slave module instantiation and connections
  --============================================================================
--
--   -- xbar_slave_out(c_MASTER_I2C_SLAVE).stall;
--   -- xbar_slave_out(c_MASTER_I2C_SLAVE).int;
--   -- xbar_master_in(c_SLAVE_I2C_SLAVE).stall,
--   -- xbar_master_in(c_SLAVE_I2C_SLAVE).int,
--
  cmp_i2c_slave: i2c_slave_top
  port map
  (
    sda_oen           => i2c_slave_o.SDA_OE,
    sda_i             => i2c_slave_i.SDA_I,
    sda_o             => i2c_slave_o.SDA_O,
    scl_oen           => i2c_slave_o.SCL_OE,
    scl_i             => i2c_slave_i.SCL_I,
    scl_o             => i2c_slave_o.SCL_O,
    wb_clk_i          => clk_50,
    wb_rst_i          => rst_i,
    wb_master_stb_o   => xbar_slave_in(c_MASTER_I2C_SLAVE).stb,
    wb_master_cyc_o   => xbar_slave_in(c_MASTER_I2C_SLAVE).cyc,
    wb_master_sel_o   => xbar_slave_in(c_MASTER_I2C_SLAVE).sel,
    wb_master_we_o    => xbar_slave_in(c_MASTER_I2C_SLAVE).we,
    wb_master_data_i  => xbar_slave_out(c_MASTER_I2C_SLAVE).dat,
    wb_master_data_o  => xbar_slave_in(c_MASTER_I2C_SLAVE).dat,
    wb_master_addr_o  => xbar_slave_in(c_MASTER_I2C_SLAVE).adr(15 downto 0),
    wb_master_ack_i   => xbar_slave_out(c_MASTER_I2C_SLAVE).ack,
    wb_master_rty_i   => xbar_slave_out(c_MASTER_I2C_SLAVE).rty,
    wb_master_err_i   => xbar_slave_out(c_MASTER_I2C_SLAVE).err,
    wb_slave_stb_i    => xbar_master_out(c_SLAVE_I2C_SLAVE).stb,
    wb_slave_cyc_i    => xbar_master_out(c_SLAVE_I2C_SLAVE).cyc,
    wb_slave_sel_i    => xbar_master_out(c_SLAVE_I2C_SLAVE).sel,
    wb_slave_we_i     => xbar_master_out(c_SLAVE_I2C_SLAVE).we,
    wb_slave_data_i   => xbar_master_out(c_SLAVE_I2C_SLAVE).dat,
    wb_slave_data_o   => xbar_master_in(c_SLAVE_I2C_SLAVE).dat,
    wb_slave_addr_i   => xbar_master_out(c_SLAVE_I2C_SLAVE).adr( 5 downto 2),
    wb_slave_ack_o    => xbar_master_in(c_SLAVE_I2C_SLAVE).ack,
    wb_slave_rty_o    => xbar_master_in(c_SLAVE_I2C_SLAVE).rty,
    wb_slave_err_o    => xbar_master_in(c_SLAVE_I2C_SLAVE).err,
    pf_wb_addr_o      => open,
    rd_done_o         => i2c_rd_done,
    wr_done_o         => i2c_wr_done,
    i2c_addr_i        => i2c_addr
  );

  xbar_slave_in(c_MASTER_I2C_SLAVE).adr(31 downto 16) <= (others => '0');

  -- Process to set the I2C_UP signal for display on the front panel
  -- of the front module. The I2C_UP signal is permanently set once an
  -- I2C transfer has successfully completed, as signaled by the RD_DONE
  -- and WR_DONE outputs of the I2C slave.
  p_i2c_up: process (clk_50) is
  begin
    if rising_edge(clk_50) then
      if (rst_n = '0') then
        i2c_up <= '0';
      elsif (i2c_rd_done = '1') or (i2c_wr_done = '1') then
        i2c_up <= '1';
      end if;
    end if;
  end process p_i2c_up;



--
--
--   inst_m25p32: m25p32_top
--   port map(wb_rst_i          => s_rst.SYS_A(c_RST_A_CLKS - 1),
--            wb_clk            => clk_50,
--            wb_we_i           => xbar_master_out(c_SLAVE_M25P32).we,
--            wb_stb_i          => xbar_master_out(c_SLAVE_M25P32).stb,
--            wb_cyc_i          => xbar_master_out(c_SLAVE_M25P32).cyc,
--            wb_sel_i          => xbar_master_out(c_SLAVE_M25P32).sel,
--            wb_data_i         => xbar_master_out(c_SLAVE_M25P32).dat,
--            wb_data_o         => xbar_master_in(c_SLAVE_M25P32).dat,
--            wb_addr_i         => xbar_master_out(c_SLAVE_M25P32).adr( 8 downto 2),
--            wb_ack_o          => xbar_master_in(c_SLAVE_M25P32).ack,
--            wb_rty_o          => xbar_master_in(c_SLAVE_M25P32).rty,
--            wb_err_o          => xbar_master_in(c_SLAVE_M25P32).err,
--            miso_word_rcv     => open,
--            op_finished_o     => open,
--            prom_mosi_o       => spi_master_o.MOSI,
--            prom_cclk_o       => spi_master_o.CCLK,
--            prom_cs0_b_n_o    => spi_master_o.CSO_B_N,
--            prom_din_i        => spi_master_i.DIN);
--
--
--   inst_multiboot: multiboot_top
--   port map(wb_rst_i  => s_rst.SYS_A(c_RST_A_CLKS - 1),
--            wb_clk    => clk_50,
--            wb_we_i   => xbar_master_out(c_SLAVE_MULTIBOOT).we,
--            wb_stb_i  => xbar_master_out(c_SLAVE_MULTIBOOT).stb,
--            wb_cyc_i  => xbar_master_out(c_SLAVE_MULTIBOOT).cyc,
--            wb_sel_i  => xbar_master_out(c_SLAVE_MULTIBOOT).sel,
--            wb_data_i => xbar_master_out(c_SLAVE_MULTIBOOT).dat,
--            wb_data_o => xbar_master_in(c_SLAVE_MULTIBOOT).dat,
--            wb_addr_i => xbar_master_out(c_SLAVE_MULTIBOOT).adr( 5 downto 2),
--            wb_ack_o  => xbar_master_in(c_SLAVE_MULTIBOOT).ack,
--            wb_rty_o  => xbar_master_in(c_SLAVE_MULTIBOOT).rty,
--            wb_err_o  => xbar_master_in(c_SLAVE_MULTIBOOT).err);
--
--

  --============================================================================
  -- Instantiation and connection of a Wishbone crossbar module
  --============================================================================
  xbar_master_in(c_SLAVE_I2C_SLAVE).stall   <= '0';
  xbar_master_in(c_SLAVE_I2C_SLAVE).int     <= '0';
  xbar_master_in(c_slave_trigleds_wb).int   <= '0';

--      xbar_master_in(c_SLAVE_M25P32).stall     <= '0';
--      xbar_master_in(c_SLAVE_M25P32).int       <= '0';
--      xbar_master_in(c_SLAVE_MULTIBOOT).stall  <= '0';
--      xbar_master_in(c_SLAVE_MULTIBOOT).int    <= '0';

  cmp_wb_crossbar: xwb_crossbar
  generic map
  (
    g_num_masters => c_NUM_MASTERS,
    g_num_slaves  => c_NUM_SLAVES,
    g_registered  => false,
    -- Address of the slaves connected
    -- It should be noted that the default address length is 32
    -- In our project only 16 bits are addressable
    g_address     => c_addresses,
    g_mask        => c_masks
  )
  port map
  (
    clk_sys_i => clk_50,
    rst_n_i   => rst_n,
    slave_i   => xbar_slave_in,
    slave_o   => xbar_slave_out,
    master_i  => xbar_master_in,
    master_o  => xbar_master_out
  );

  --============================================================================
  -- I2C test trigger LEDs instantiation and connections
  --============================================================================
  cmp_test_trigleds: test_trigleds_wb
  port map
  (
    rst_n_i             => rst_n,
    clk_sys_i           => clk_50,
    wb_dat_i            => xbar_master_out(c_slave_trigleds_wb).dat,
    wb_dat_o            => xbar_master_in(c_slave_trigleds_wb).dat,
    wb_cyc_i            => xbar_master_out(c_slave_trigleds_wb).cyc,
    wb_sel_i            => xbar_master_out(c_slave_trigleds_wb).sel,
    wb_stb_i            => xbar_master_out(c_slave_trigleds_wb).stb,
    wb_we_i             => xbar_master_out(c_slave_trigleds_wb).we,
    wb_ack_o            => xbar_master_in(c_slave_trigleds_wb).ack,
    wb_stall_o          => xbar_master_in(c_slave_trigleds_wb).stall,
    -- Port for std_logic_vector field: 'Bits' in reg: 'LED'
    trigleds_reg_bits_o => trigleds
  );   

  -- !!!!!
  led_front_n <= not trigleds;

  --============================================================================
  -- RTM detection module
  --============================================================================
  cmp_rtm_detector: rtm_detector
  port map
  (
    rtmm_i    => rtmm_i,
    rtmp_i    => rtmp_i,
    rtmm_ok_o => rtmm_ok,
    rtmp_ok_o => rtmp_ok
  );

  --============================================================================
  -- Bicolor LED matrix controller
  --============================================================================
  leds_array.top.pwr        <= '1';
  leds_array.middle.wr_ok   <= '0';
  leds_array.middle.wr_link <= '0';
  leds_array.middle.wr_addr <= '0';
  
  -- !!! Should be OR function on V2 RTMPs
  leds_array.top.err        <= '1' when (rtmm_ok = '0') and (rtmp_ok = '0') else
                               '0';
   
  -- Here are organized in the same disposition as in the front panel.
  -- Take a look to image1_led_pkg.vhd for the correct order for
  -- bicolor_led_ctrl
  led_state_array(c_LED_NB_PWR)        <= f_led_state(c_LED_COLOR_PWR) when leds_array.top.PWR = '1' else
                                          f_led_state(c_LED_OFF);
  led_state_array(c_LED_NB_ERR)        <= f_led_state(c_LED_RED) when leds_array.top.ERR = '1' else
                                          f_led_state(c_LED_OFF);
  led_state_array(c_LED_NB_TTL_N)      <= f_led_state(c_LED_COLOR_TTL_N) when leds_array.top.TTL_N = '1' else 
                                          f_led_state(c_LED_OFF);
  led_state_array(c_LED_NB_I2C)        <= f_led_state(c_LED_COLOR_I2C) when i2c_up = '1' else 
                                          f_led_state(c_LED_RED);
  led_state_array(c_LED_NB_WR_OK)      <= f_led_state(c_LED_COLOR_WR_OK) when leds_array.middle.WR_OK = '1' else
                                          f_led_state(c_LED_RED);
  led_state_array(c_LED_NB_WR_LINK)    <= f_led_state(c_LED_COLOR_WR_LINK) when leds_array.middle.WR_LINK = '1' else 
                                          f_led_state(c_LED_RED);
  led_state_array(c_LED_NB_WR_GMT)     <= f_led_state(c_LED_COLOR_WR_GMT) when leds_array.middle.WR_GMT = '1'else 
                                          f_led_state(c_LED_RED);
  led_state_array(c_LED_NB_WR_ADDR)    <= f_led_state(c_LED_COLOR_WR_ADDR) when leds_array.middle.WR_ADDR = '1' else 
                                          f_led_state(c_LED_RED);
  led_state_array(c_LED_NB_MULTICAST3) <= f_led_state(c_LED_COLOR_MULTICAST3) when leds_array.bottom.MULTICAST(3) = '1' else 
                                          f_led_state(c_LED_OFF);
  led_state_array(c_LED_NB_MULTICAST2) <= f_led_state(c_LED_COLOR_MULTICAST2) when leds_array.bottom.MULTICAST(2) = '1' else 
                                          f_led_state(c_LED_OFF);
  led_state_array(c_LED_NB_MULTICAST1) <= f_led_state(c_LED_COLOR_MULTICAST1) when leds_array.bottom.MULTICAST(1) = '1' else 
                                          f_led_state(c_LED_OFF);
  led_state_array(c_LED_NB_MULTICAST0) <= f_led_state(c_LED_COLOR_MULTICAST0) when leds_array.bottom.MULTICAST(0) = '1' else 
                                          f_led_state(c_LED_OFF);

  cmp_bicolor_led_ctrl: bicolor_led_ctrl
  generic map
  (
    g_NB_COLUMN    => c_NB_COLUMN,
    g_NB_LINE      => c_NB_LINE,
    g_CLK_FREQ     => 20000000,
    g_REFRESH_RATE => 250
  )
  port map
  (
    rst_n_i         => rst_n,
    clk_i           => clk_20_i,
    led_intensity_i => c_LED_INTENSITY,
    led_state_i     => f_STD_LOGIC_VECTOR(led_state_array),
    column_o(0)     => led_array_o.WR_OWNADDR_I2C,
    column_o(1)     => led_array_o.WR_GMT_TTL_TTLN,
    column_o(2)     => led_array_o.WR_LINK_SYSERROR,
    column_o(3)     => led_array_o.WR_OK_SYSPW,
    column_o(4)     => led_array_o.MULTICAST_2_0,
    column_o(5)     => led_array_o.MULTICAST_3_1,
    line_o(0)       => led_array_o.CTRL0,
    line_o(1)       => led_array_o.CTRL1,
    line_oen_o(0)   => led_array_o.CTRL0_OEN,
    line_oen_o(1)   => led_array_o.CTRL1_OEN
  );

end Behavioral;
