----------------------------------------------------------------------------------
-- Company:        CERN, BE-CO-HT
-- Engineer:       Carlos Gil Soriano
--                 Theodor-Adrian Stana
--
-- Create Date:    09:58:06 10/12/2011
-- Design Name:    HDL trigger top
-- Module Name:    trigger - Behavioral
-- Project Name:   CTDAH
-- Target Devices: Spartan 6
-- Tool versions:
-- Description:    This is the wishbone trigger which receives a input
--                 and outputs a trigger signal for the pulse converter.
--                 It internally debounces the inputs and control the output
--                 driver.
--                 The registers it has can be modified via wishbone access.
-- Dependencies:   none
--
-- Revision:
-- Revision 0.1
-- Additional Comments:
--
----------------------------------------------------------------------------------
library IEEE;
library unisim;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use UNISIM.VCOMPONENTS.ALL;

entity basic_trigger_top is
  generic
  (
    g_number_of_channels   : natural := 6;
    g_clk_period           : time    := 20 ns;
    g_pulse_length         : time    := 1000 ns;
    g_led_blinking_length  : time    := (10**6)*250 ns
  );
  port 
  (
    clk_i          : in  std_logic;
    rst_i          : in  std_logic;
    led_ttl_o      : out std_logic;
    
    -- Enable signals
    fpga_o_en      : out std_logic;
    fpga_o_ttl_en  : out std_logic;
    fpga_o_inv_en  : out std_logic;
    fpga_o_blo_en  : out std_logic;
    level_i        : in  std_logic;
    switch_i       : in  std_logic;
    
    -- The manual reset allows power sequencing of the
    -- 24V rail after a security given delay
    manual_rst_n_o : out std_logic;
    
    -- Front and rear pulse signals
    pulse_i_front  : in  std_logic_vector(g_number_of_channels downto 1);
    pulse_o_front  : out std_logic_vector(g_number_of_channels downto 1);
    pulse_i_rear   : in  std_logic_vector(g_number_of_channels downto 1);
    pulse_o_rear   : out std_logic_vector(g_number_of_channels downto 1);
    
    -- Front and rear LED signals
    led_o_front    : out std_logic_vector(g_number_of_channels downto 1);
    led_o_rear     : out std_logic_vector(g_number_of_channels downto 1);

    -- Inverting inputs and outputs
    inv_i          : in  std_logic_vector(4 downto 1);
    inv_o          : out std_logic_vector(4 downto 1)
  );
end basic_trigger_top;


architecture Behavioral of basic_trigger_top is

  --============================================================================
  -- Component declarations
  --============================================================================
  component basic_trigger_core is
    generic
    (
      g_clk_period           : time;
      g_output_pulse_length  : time;
      g_led_blinking_length  : time
    );
    port 
    (
      wb_rst_i  : in  std_logic;
      wb_clk_i  : in  std_logic;

      pulse_i   : in  std_logic;
      pulse_o   : out std_logic;
      pulse_n_o : out std_logic;

      crop_o    : out std_logic;

      led_o     : out std_logic
    );
  end component;
  
  --============================================================================
  -- Signal declarations
  --============================================================================
  signal pulse_in        : std_logic_vector(g_number_of_channels downto 1);
  signal pulse_in_front  : std_logic_vector(g_number_of_channels downto 1);
  signal pulse_out       : std_logic_vector(g_number_of_channels downto 1);
  signal pulse_out_n     : std_logic_vector(g_number_of_channels downto 1);
  signal led             : std_logic_vector(g_number_of_channels downto 1);

  signal crop            : std_logic_vector(g_number_of_channels downto 1) := (others => '0');
  signal level           : std_logic;

  signal fpga_out_en     : std_logic;
  signal fpga_out_ttl_en : std_logic;
  signal fpga_out_inv_en : std_logic;
  signal fpga_out_blo_en : std_logic;

  -- type delay_array is array (g_number_of_channels downto 1) of std_logic_vector(3 downto 0);
  -- signal pulse_in_reg : delay_array;

begin

  -- Level selection via on-board switch: 
  -- +---------+-----------------+
  -- | level_i |  Switch | Level |
  -- +---------+---------+-------+
  -- | 0 TTL   |    UP   |  TTL  |
  -- | 1 TTL_N |   DOWN  | TTL_N |
  -- +---------+---------+-------+
  level     <= level_i;
  led_ttl_o <= level;

  -- pulse signal assignments
  pulse_in_front <= pulse_i_front when (level = '0') else 
                    not pulse_i_front;
  pulse_in       <= pulse_in_front or pulse_i_rear;

  fpga_o_en       <= fpga_out_en when (switch_i = '0') else '0';
  fpga_o_ttl_en   <= fpga_out_ttl_en;
  fpga_o_inv_en   <= fpga_out_inv_en;
  fpga_o_blo_en   <= fpga_out_blo_en;

  led_o_front     <= not(led); -- No need of accurate sync, hence we place
  led_o_rear      <= not(led); -- some combinatorial here.

  pulse_o_front   <= pulse_out  when level = '0' else
                     not pulse_out;
  pulse_o_rear    <= pulse_out;

  -- As we have one Schmitt inverter in the input,
  -- and a buffer in the output, there's no need
  -- to invert here.
  inv_o <= inv_i;
  
  gen_trig_cores: for i in 1 to g_number_of_channels generate
    trigger: basic_trigger_core
    generic map
    (
      g_clk_period          => g_clk_period,
      g_output_pulse_length => g_pulse_length,
      g_led_blinking_length => g_led_blinking_length
    )
    port map 
    (
      wb_rst_i  => rst_i,
      wb_clk_i  => clk_i,
      pulse_i   => pulse_in(i),
      pulse_o   => pulse_out(i),
      pulse_n_o => pulse_out_n(i),
      crop_o    => open,
      led_o     => led(i)
    );
  end generate gen_trig_cores;

  -- Process to lock the enables so to avoid output glitches
  -- on the startup.
  p_reset_chain : process(clk_i) is
  begin
    if rising_edge(clk_i) then
      if (rst_i = '1') then
        -- First we reset the FPGA general output enable
        -- Then we let one clock delay for the rest of signals
        manual_rst_n_o   <= '0';
        fpga_out_en      <= '0';
        fpga_out_ttl_en  <= '0';
        fpga_out_inv_en  <= '0';
        fpga_out_blo_en  <= '0';
      else
        manual_rst_n_o   <= '1';
        fpga_out_en      <= '1';
        fpga_out_ttl_en  <= fpga_out_en;
        fpga_out_inv_en  <= fpga_out_en;
        fpga_out_blo_en  <= fpga_out_en;
      end if;
    end if;
  end process p_reset_chain;

end Behavioral;
