--------------------------------------------------------------------------------
-- Company:        CERN, BE-CO-HT
-- Engineer:       Carlos Gil Soriano
--
-- Create Date:    11:56:55 10/25/2011
-- Design Name:    I2C slave register HDL
-- Module Name:    i2c_regs - Behavioral
-- Project Name:   Level Conversion Circuits
-- Target Devices: Spartan 6
-- Tool versions:
-- Description:    This is the HDL that holds all the register of the I2C slave
--                 core
--
-- The registers should be programmed in the following order:
--   1.- CTR0 Sets up the control register: enable, start, i2c address
--   2.- CTR1 Sets up the threshold for detection signals in RX and TX buffers
--
-- Revision:
-- Revision 0.01 - File Created
--          1.0  - Stable version
-- Additional Comments:
--
----------------------------------------------------------------------------------
library IEEE;
library work;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.i2c_slave_pkg.ALL;

entity i2c_regs is
  port 
  (
    wb_clk_i           : in   STD_LOGIC;
    wb_rst_i         : in   STD_LOGIC;

    wb_master_we_o   : out STD_LOGIC;
    wb_master_stb_o  : out STD_LOGIC;
    wb_master_cyc_o  : out STD_LOGIC;
    wb_master_sel_o  : out STD_LOGIC_VECTOR (3  downto 0);
    wb_master_data_i : in  STD_LOGIC_VECTOR (31 downto 0);
    wb_master_data_o : out STD_LOGIC_VECTOR (31 downto 0);
    wb_master_addr_o : out STD_LOGIC_VECTOR (15  downto 0);
    wb_master_ack_i  : in  STD_LOGIC;
    wb_master_rty_i  : in  STD_LOGIC;
    wb_master_err_i  : in  STD_LOGIC;

    -- These are the registers offers to others modules of the FPGA
    wb_slave_we_i   : in   STD_LOGIC;
    wb_slave_stb_i  : in   STD_LOGIC;
    wb_slave_cyc_i  : in   STD_LOGIC;
    wb_slave_sel_i  : in   STD_LOGIC_VECTOR (3  downto 0);
    wb_slave_data_i : in   STD_LOGIC_VECTOR (31 downto 0);
    wb_slave_data_o : out  STD_LOGIC_VECTOR (31 downto 0);
    wb_slave_addr_i : in   STD_LOGIC_VECTOR (3  downto 0);
    wb_slave_ack_o  : out  STD_LOGIC;
    wb_slave_rty_o  : out  STD_LOGIC;
    wb_slave_err_o  : out  STD_LOGIC;

    -- These are the registers that are offered to the i2c slave core
    CTR0_o          : out  STD_LOGIC_VECTOR (r_CTR0'a_length - 1 downto 0);
    LT_i            : in   STD_LOGIC_VECTOR (r_LT'a_length -1  downto 0);
    DRXA_i          : in   STD_LOGIC_VECTOR (r_DRX'a_length  - 1 downto 0);
    DRXB_i          : in   STD_LOGIC_VECTOR (r_DRX'a_length  - 1 downto 0);

    pf_wb_addr_i    : in  STD_LOGIC;
    pf_wb_data_o    : out STD_LOGIC_VECTOR(31 downto 0);
    rd_done_i       : in  STD_LOGIC;
    wr_done_i       : in  STD_LOGIC;
    i2c_addr_i      : in  STD_LOGIC_VECTOR(6 downto 0)
  );
end i2c_regs;

architecture Behavioral of i2c_regs is

  type t_wb_state is (
                       R0_RESET,
                       S0_IDLE,
                       S1P_WB_RD_RQT, --! Prefetch
                       S1_WB_RD_RQT,
                       S1N_WB_NOOP,
                       S1_PF_WB_DATA_OUT,
                       S2P_WB_WR_RQT, --! Prefetch
                       S2_WB_WR_RQT,
                       S2N_WB_NOOP,
                       S3_WB_ACK
                     );

  signal wb_state         : t_wb_state := R0_RESET;

  signal s_wb_slave_addr  : UNSIGNED(3 downto 0);
  signal s_wb_slave_ack   : STD_LOGIC := '0';
  signal s_wb_slave_rty   : STD_LOGIC := '0';
  signal s_wb_slave_err   : STD_LOGIC := '0';

  signal s_CTR0_slv       : STD_LOGIC_VECTOR (r_CTR0'a_length - 1 downto 0);
  signal s_CTR0           : r_CTR0 := c_CTR0_default;
  signal s_LT             : r_LT;
  signal s_DTX            : STD_LOGIC_VECTOR (r_DTX'a_length  - 1 downto 0);

  signal s_wb_master_we_o : STD_LOGIC;

  signal s_wb_master_ack_retries : STD_LOGIC_VECTOR(c_RETRY_LENGTH - 1 downto 0) := (others => '0');
  signal s_wb_addr_rd      : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');

begin

  wb_master_we_o   <= s_wb_master_we_o;
  s_wb_slave_addr  <= UNSIGNED(wb_slave_addr_i);
  s_CTR0_slv       <= f_STD_LOGIC_VECTOR(s_CTR0);
  s_LT             <= f_LT(LT_i);

  wb_slave_ack_o  <=  s_wb_slave_ack;
  wb_slave_rty_o  <=  s_wb_slave_rty;
  wb_slave_err_o  <=  s_wb_slave_err;

  CTR0_o          <= s_CTR0_slv;
  pf_wb_data_o    <= s_DTX;

  --! @brief          Process that controls the retries of the wishbone interface
  --! @param wb_clk_i   Main clock
  p_wb_master_retries: process (wb_clk_i)
  begin
    if rising_edge(wb_clk_i) then
      if wb_state = S3_WB_ACK then
        s_wb_master_ack_retries(0) <= '1';
        for i in 1 to c_RETRY_LENGTH - 1 loop
          s_wb_master_ack_retries(i) <= s_wb_master_ack_retries(i-1);
        end loop;
      else
        s_wb_master_ack_retries <= (others => '0');
      end if;
    end if;
  end process;

  -- Process that outputs the wishbone
  -- master interface. It should be noted
  -- that we double-buffer the received data
  -- to assure consistency of the data to be
  -- read/written when repetitions of the
  -- wishbone master interface happen.
  p_wb_master: process(wb_clk_i)
  begin
    if rising_edge(wb_clk_i) then
      if (wb_rst_i = '1') then
        s_wb_master_we_o  <= '0';
        wb_master_stb_o   <= '0';
        wb_master_cyc_o   <= '0';
        wb_master_sel_o   <= (others => '0');
        wb_master_data_o       <= (others => '0');
        wb_master_addr_o  <= (others => '0');
        s_dtx             <= (others => '0');
      else
        case wb_state is
          when R0_RESET =>
            null;

          when S0_IDLE =>
            null;

          when S1P_WB_RD_RQT =>
            s_wb_addr_rd     <= DRXA_i(23 downto 8);

          when S1_WB_RD_RQT =>
            wb_master_cyc_o  <= '1';
            wb_master_stb_o  <= '1';
            wb_master_sel_o  <= X"F";
            wb_master_addr_o <= s_wb_addr_rd;

          when S1N_WB_NOOP =>
            wb_master_cyc_o  <= '1';
            wb_master_stb_o  <= '1';
            wb_master_sel_o  <= X"F";
            wb_master_addr_o <= s_wb_addr_rd;

          when S1_PF_WB_DATA_OUT =>
            s_DTX            <= wb_master_data_i;

          when S2P_WB_WR_RQT =>
            null;

          when S2_WB_WR_RQT   =>
            wb_master_cyc_o  <= '1';
            wb_master_stb_o  <= '1';
            s_wb_master_we_o <= '1';
            wb_master_sel_o  <= X"F";
            wb_master_data_o <= f_ch_endian(DRXA_i);
            wb_master_addr_o <= DRXB_i(15 downto 0);

          when S2N_WB_NOOP =>
            wb_master_cyc_o  <= '1';
            wb_master_stb_o  <= '1';
            s_wb_master_we_o <= '1';
            wb_master_sel_o  <= X"F";
            wb_master_data_o <= f_ch_endian(DRXA_i);
            wb_master_addr_o <= DRXB_i(15 downto 0);

          when S3_WB_ACK =>
            -- null;
            wb_master_cyc_o  <= '0';
            wb_master_stb_o  <= '0';
            s_wb_master_we_o <= '0';
          when others =>
            null;
        end case;
      end if;
    end if;
  end process p_wb_master;

  -- Process to rule slave wishbone outputs
  p_wb_slave: process (wb_clk_i)
  begin
    if rising_edge(wb_clk_i) then
      if wb_state = R0_RESET then
        s_CTR0           <= c_CTR0_default;
        s_CTR0.I2C_ADDR  <= UNSIGNED(i2c_addr_i);
        s_wb_slave_ack   <= '0';
        s_wb_slave_rty   <= '0';
        s_wb_slave_err   <= '0';
      else
        s_wb_slave_ack   <= '0';
        s_wb_slave_rty   <= '0';
        s_wb_slave_err   <= '0';
        if (wb_slave_stb_i = '1') and (wb_slave_cyc_i = '1') and
           (s_wb_slave_ack = '0') and (s_wb_slave_rty = '0') and
           (s_wb_slave_err = '0') then
          case wb_slave_we_i is
            when '1' =>
              s_wb_slave_ack           <= '1';
              case s_wb_slave_addr is
                -- when c_CTR0_addr =>
                --    s_CTR0    <= f_CTR0(wb_slave_data_i);
                when others    =>
                  s_wb_slave_ack           <= '0';
                  s_wb_slave_err           <= '1';
              end case;
            when others =>
              s_wb_slave_ack           <= '1';
              case s_wb_slave_addr is
                when c_CTR0_addr     =>
                  wb_slave_data_o <= f_STD_LOGIC_VECTOR(s_CTR0);
                when c_LT_addr       =>
                  wb_slave_data_o <= f_STD_LOGIC_VECTOR(s_LT);
                when c_DTX_addr      =>
                  wb_slave_data_o <= s_DTX;
                when c_DRXA_addr     =>
                  wb_slave_data_o <= DRXA_i;
                when c_DRXB_addr     =>
                  wb_slave_data_o <= DRXB_i;
                when others          =>
                  s_wb_slave_ack           <= '0';
                  s_wb_slave_err           <= '1';
              end case;
          end case;
        end if;
      end if;
    end if;
  end process;


  -- This is the process that controls the wishbone master interface
  -- which bridges the i2c interface with the wishbone interface.
  -- wb_clk_i Main clock
  p_master_fsm: process(wb_clk_i)
  begin
    if rising_edge(wb_clk_i) then
      if wb_rst_i = '1' then
        -- Here is the part in which we propagate the reset, let one-clock
        -- for it.
        wb_state  <=  R0_RESET;
      else
        case wb_state is
          when R0_RESET =>
            wb_state    <= S0_IDLE;
          when S0_IDLE =>
            if pf_wb_addr_i = '1' then
              wb_state <= S1P_WB_RD_RQT;
            elsif rd_done_i = '1' then
              wb_state <= S2P_WB_WR_RQT;
            end if;
          when S1P_WB_RD_RQT =>
            wb_state <= S1_WB_RD_RQT;
          when S1_WB_RD_RQT =>
            wb_state <= S1N_WB_NOOP;
          when S1N_WB_NOOP =>
            wb_state <= S1_PF_WB_DATA_OUT;
          when S1_PF_WB_DATA_OUT =>
            wb_state <= S3_WB_ACK;
          when S2P_WB_WR_RQT =>
            wb_state <= S2_WB_WR_RQT;
          when S2_WB_WR_RQT =>
            wb_state <= S2N_WB_NOOP;
          when S2N_WB_NOOP =>
            wb_state <= S3_WB_ACK;
          when S3_WB_ACK =>
            wb_state <= R0_RESET;
            if wb_master_ack_i = '1' then
              wb_state <= R0_RESET;
            else
              if wb_master_rty_i = '1' then
                if s_wb_master_ack_retries(c_RETRY_LENGTH - 1) = '1' then
                  -- We stop retrying to not block the core
                  wb_state <= R0_RESET;
                else
                  if s_wb_master_we_o = '1' then
                    wb_state <= S1_WB_RD_RQT;
                  else
                    wb_state <= S2_WB_WR_RQT;
                  end if;
                end if;
              end if;
            end if;
          when others =>
            wb_state <= R0_RESET;
        end case;
      end if;
    else
    end if;
  end process p_master_fsm;

end Behavioral;
