----------------------------------------------------------------------------------
-- Company:        CERN, BE-CO-HT
-- Engineers:      Carlos Gil Soriano
--                 Theodor-Adrian Stana
--
-- Create Date:    11:29:56 10/25/2011
-- Design Name:    I2C Slave to Wishbone bridge
-- Module Name:    i2c_slave_core - Behavioral
-- Project Name:   Level Conversion Circuits
-- Target Devices: Spartan 6
-- Tool versions:
-- Description:    This is a bridge from I2C to Wishbone.
--                 The module consist on the following interfaces:
--                    - I2C interface
--                    - Wishbone master
--                    - Wishbone slave
--                    - Interruption lines
--                 Each time a valid instruction is set in the I2C interface, this
--                 is reported either:
--                    A.- Via Wishbone: acting the module as a Wishbone master.
--                    B.- Interrupts:   the module offers some interrupt pins that
--                                      allows an external processor interact with
--                                      the module via the Wishbone slave
--                                      interface.
--                 The FSM of this I2C core is able to detect indirect I2C
--                 addressing. Thus reads, writes and prefetching of data
--                 through Wishbone master interface will be issued.
--                 ~ SUBCOMPONENTS ~
--                    This component evaluates the data at the byte level, upon
--                 the previous analysis of the SCL and SDA pins at a bit level.
--                 The bit level analysis is carried out in:
--                       --> ./i2c_bit.vhd
--                    Two internal FIFOs (one for RX and another TX) are combined
--                 with this module to store the received and ready-to-be sent
--                 frames. The files are:
--                       --> ./FIFO_stack.vhd
--                       --> ./FIFO_dispatcher.vhd
--                    Both the Wishbone master and slave interface access control
--                 and all configuration registers can be found in:
--                       --> ./i2c_regs.vhd
--
-- Dependencies:   i2c_bit.vhd, FIFO_stack.vhd, FIFO_dispatcher.vhd, gc_ff.vhd
--
-- Revision:
-- Revision 0.01 - File Created
--          0.1  - Initial submission. I2C instructions for Read and Write work.
--          1.0  - Code completely revamped. Code tested with a code coverage of
--                 81.5% with the testbench provided in:
--                       ../test/i2c_slave_test_top_tb.vhd
--
-- Additional Comments:
--
----------------------------------------------------------------------------------
library IEEE;
library work;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.i2c_slave_pkg.ALL;
use work.ctdah_pkg.ALL;

entity i2c_slave_core is
  generic
  (
    g_WB_CLK_PERIOD : time := 50 ns
  );
  port 
  (
    clk_i         : in  std_logic;
    rst_i         : in  std_logic;
    -- I2C pins
    sda_oen       : out  std_logic;
    sda_i         : in   std_logic;
    sda_o         : out  std_logic;
    scl_oen       : out  std_logic;
    scl_i         : in   std_logic;
    scl_o         : out  std_logic;
    
    -- Registers
    CTR0_i        : in  std_logic_vector (r_CTR0'a_length - 1 downto 0);
    LT_o          : out std_logic_vector (r_LT'a_length   - 1 downto 0);
    DRXA_o        : out std_logic_vector (r_DRX'a_length  - 1 downto 0);
    DRXB_o        : out std_logic_vector (r_DRX'a_length  - 1 downto 0);

    -- Alarms for controlling the i2c states
    pf_wb_addr_o  : out  std_logic;
    pf_wb_data_i  : in   std_logic_vector(31 downto 0);
    rd_done_o     : out  std_logic;
    wr_done_o     : out  std_logic
  );
end i2c_slave_core;



architecture Behavioral of i2c_slave_core is

  type t_state is (
                    R0_RESET,
                    S0_IDLE,
                    S1_START_DETECT,
                    S2_I2C_ADDR,
                    S2A_I2C_ADDR_ACK,
                    S2NA_I2C_ADDR_NACK,
                    S3_WISHBONE_ADDR,
                    S3A_WISHBONE_ADDR_ACK,
                    S4_DETECT_OPERATION,
                    S5R_READ_SDA,
                    S5RA_READ_SDA_ACK,
                    S5W0_RESTART,
                    S5W1_I2C_ADDR,
                    S5W1A_I2C_ADDR_ACK,
                    S5W1NA_I2C_ADDR_NACK,
                    S5W2_WRITE_SDA,
                    S5W2A_WRITE_SDA_ACK,
                    S5W3_NACK,
                    S6_WAIT_START_PAUSE,
                    S7_PAUSE_DETECT
                 );

  -------------------------------------------------------------------------------
  -- FSM signals for the I2C module
  -------------------------------------------------------------------------------
  -- Four signals are employed to govern the FSM behaviour and its outputs.
  --
  -- These signals are updated in done_comb_proc process, dependant of the done
  -- signal coming from i2c_bit.vhd
  -- Every time a deglitched falling edge of the SCL line is detected in the
  -- aforementioned i2c_bit.vhd, the signals done, start_o, pause_o, rcved_o are
  -- considered valid to be studied in i2c_slave_core.vhd processes.
  --
  -------------------------------------------------------------------------------

  constant c_WATCHDOG_END_VALUE : NATURAL := c_WATCHDOG_DEADLINE/g_WB_CLK_PERIOD;

  signal state            : t_state := R0_RESET;

  signal state_d0         : t_state := R0_RESET;

  signal s_DRXA_slv  : std_logic_vector(r_DRX'a_length - 1 downto 0) := (others => '0');
  signal s_DRXB_slv  : std_logic_vector(r_DRX'a_length - 1 downto 0) := (others => '0');
  signal s_DRX_slv   : std_logic_vector(r_DRX'a_length*2 - 1 downto 0) := (others => '0');
  signal s_DTX_slv   : std_logic_vector(r_DTX'a_length - 1 downto 0);

  signal s_CTR0      : r_CTR0;
  signal s_LT        : r_LT  := c_LT_default;

  signal s_i2c_addr_ctr0  : std_logic_vector(7 downto 0);
  signal s_i2c_addr       : std_logic_vector(7 downto 0);

  -------------------------------------------------------------------------------
  -- i2c signals
  -------------------------------------------------------------------------------
  signal s_sda_o            : std_logic;
  signal s_sda_oen          : std_logic;

  signal s_bit_done         : std_logic;

  -- Bit counter signals
  signal s_bit_cnt_slv      : std_logic_vector(c_COUNTER_WIDTH - 1 downto 0);
  signal s_bit_cnt          : unsigned(7 downto 0);
  signal s_bit_cnt_rst      : std_logic;

  -- Byte counter signals
  signal s_byte_cnt_slv     : std_logic_vector(c_COUNTER_WIDTH - 1 downto 0);
  signal s_byte_cnt         : unsigned(7 downto 0);
  signal s_byte_cnt_rst     : std_logic;
  signal s_byte_cnt_en      : std_logic := '0';

  -- Interrupting signals (i2c_bit.vhd)
  signal s_start_o          : std_logic;
  signal s_pause_o          : std_logic;
  signal s_rcved_o          : std_logic;

  signal s_pf_wb_data       : std_logic_vector(31 downto 0) := (others => '1');
  signal s_pf_wb_data_arr   : std_logic_vector(31 downto 0) := (others => '1');

  signal s_watchdog_cnt_rst : std_logic;
  signal s_watchdog_cnt_slv : std_logic_vector(c_WATCHDOG_WIDTH - 1 downto 0);
  signal s_watchdog_cnt     : NATURAL;

  signal txsr : std_logic_vector(31 downto 0);

begin

  s_CTR0       <= f_CTR0(CTR0_i);
  LT_o         <= f_STD_LOGIC_VECTOR(s_LT);
  s_pf_wb_data <= pf_wb_data_i;

  cmp_i2c_bit: i2c_bit
  port map
  (
    rst_i     => rst_i,
    wb_clk_i  => clk_i,
    sda_i     => sda_i,
    scl_i     => scl_i,
    start_o   => s_start_o,
    pause_o   => s_pause_o,
    rcved_o   => s_rcved_o,
    done_o    => s_bit_done
  );

  cmp_bit_counter: gc_counter
  generic map 
  (
    g_data_width => c_DATA_WIDTH
  )
  port map 
  (
    clk_i  => clk_i,
    rst_i  => s_bit_cnt_rst,
    en_i   => s_bit_done,
    cnt_o  => s_bit_cnt_slv
  );

  s_bit_cnt  <= unsigned(s_bit_cnt_slv);

  cmp_byte_counter: gc_counter
  generic map 
  (
    g_data_width => c_DATA_WIDTH
  )
  port map 
  (
    clk_i  => clk_i,
    rst_i  => s_byte_cnt_rst,
    en_i   => s_byte_cnt_en,
    cnt_o  => s_byte_cnt_slv
  );

  s_byte_cnt <= unsigned(s_byte_cnt_slv);

  cmp_watchdog_counter: gc_counter
  generic map 
  (
    g_data_width => c_WATCHDOG_WIDTH
  )
  port map 
  (
    clk_i  => clk_i,
    rst_i  => s_watchdog_cnt_rst,
    en_i   => c_WATCHDOG_ENABLE,
    cnt_o  => s_watchdog_cnt_slv
  );

  s_watchdog_cnt <= to_integer(UNSIGNED(s_watchdog_cnt_slv));

  scl_o   <= '1';
  scl_oen <= '0';

  sda_o <= s_sda_o;

  p_sda_o: process(state, txsr)
  begin
    case state is
      when S2A_I2C_ADDR_ACK      =>
        s_sda_o <= '0';
      when S3A_WISHBONE_ADDR_ACK =>
        s_sda_o <= '0';
      when S5RA_READ_SDA_ACK     =>
        s_sda_o <= '0';
      when S5W1A_I2C_ADDR_ACK    =>
        s_sda_o <= '0';
      when S5W2_WRITE_SDA        =>
        s_sda_o <= txsr(31);
      when others =>
        s_sda_o <= '1';
    end case;
  end process;

  sda_oen <= s_sda_oen;

  p_sda_oen: process(state)
  begin
    s_sda_oen <= '0';
    case state is
      when S2A_I2C_ADDR_ACK      =>
        s_sda_oen <= '1';
      when S3A_WISHBONE_ADDR_ACK =>
        s_sda_oen <= '1';
      when S5RA_READ_SDA_ACK     =>
        s_sda_oen <= '1';
      when S5W1A_I2C_ADDR_ACK    =>
        s_sda_oen <= '1';
      when S5W2_WRITE_SDA        =>
        s_sda_oen <= '1';
      when others                =>
        null;
    end case;
  end process;

  s_DRXA_slv <= s_DRX_slv(r_DRX'a_length - 1    downto 0);
  s_DRXB_slv <= s_DRX_slv(r_DRX'a_length*2 - 1 downto r_DRX'a_length);

  --Process to store the incoming data
  p_store_FIFO: process(clk_i)
    
    procedure shift_DRX is
    begin
      for i in 1 to s_DRX_slv'length - 1 loop
        s_DRX_slv(i) <= s_DRX_slv(i-1);
      end loop;
      s_DRX_slv(0) <= s_rcved_o;
    end procedure;

  begin
    if rising_edge(clk_i) then
      if s_bit_done = '1' then
        case state is
          when S2_I2C_ADDR      =>
            shift_DRX;
          when S3_WISHBONE_ADDR =>
            shift_DRX;
          when S4_DETECT_OPERATION =>
            if     s_start_o = '0'
            and s_pause_o = '0' then
              shift_DRX;
            end if;
          when S5R_READ_SDA     =>
            shift_DRX;
          when S5W1_I2C_ADDR    =>
            shift_DRX;
          when others =>
            null;
        end case;
      end if;
    end if;
  end process;


  s_i2c_addr_ctr0(7 downto 1) <= std_logic_vector(s_CTR0.I2C_ADDR);
  s_i2c_addr(0)               <= '0';

  gen_i2c: for i in 1 to 7 generate
    s_i2c_addr(8-i) <= s_i2c_addr_ctr0(i);
  end generate gen_i2c;

  p_delays : process(clk_i)
  begin
    if rising_edge(clk_i) then
      state_d0   <= state;
    end if;
  end process;

   -- Process to generate the pf_wb_addr_o signal.
  p_pf_wb_addr: process(clk_i)
  begin
    if rising_edge(clk_i) then
      pf_wb_addr_o <= '0';
      if     state    = S5W1A_I2C_ADDR_ACK
      and state_d0 = S5W1_I2C_ADDR      then
        pf_wb_addr_o <= '1';
      end if;
    end if;
  end process;

  -- Process to generate the rd_done_o signal.
  p_rd_done: process(clk_i)
  begin
    if rising_edge(clk_i) then
      rd_done_o <= '0';
      if     state = S6_WAIT_START_PAUSE
      and state_d0 = S5RA_READ_SDA_ACK     then
        rd_done_o <= '1';
      end if;
    end if;
  end process;

  -- Process to generate the wr_done_o signal.
  p_wr_done: process(clk_i)
  begin
    if rising_edge(clk_i) then
      wr_done_o <= '0';
      if     state    = S6_WAIT_START_PAUSE
      and state_d0 = S5W3_NACK           then
        wr_done_o <= '1';
      end if;
    end if;
  end process;

  DRXA_o <= s_DRXA_slv;
  DRXB_o <= s_DRXB_slv;

  -- Process to update the signals that drive bit_counter_8
  p_bit_counter_comb : process(state, state_d0, s_byte_cnt)
  begin
    s_bit_cnt_rst  <= '0';
    s_byte_cnt_rst <= '0';

    case state is

      when R0_RESET =>
        if state_d0 /= R0_RESET then
          s_bit_cnt_rst  <= '1';
          s_byte_cnt_rst <= '1';
        end if;

      when S0_IDLE =>
        null;

      when S1_START_DETECT =>
        s_bit_cnt_rst  <= '1';
        s_byte_cnt_rst <= '1';

      when S2_I2C_ADDR =>
        null;

      when S2A_I2C_ADDR_ACK =>
        if (state_d0 = S2_I2C_ADDR) then
          s_byte_cnt_rst <= '1';
        end if;

      when S3_WISHBONE_ADDR =>
        if (state_d0 = S2A_I2C_ADDR_ACK) or 
           (state_d0 = S3A_WISHBONE_ADDR_ACK) or 
           (state_d0 = S5W1A_I2C_ADDR_ACK) then
          s_bit_cnt_rst  <= '1';
        end if;
        if (s_byte_cnt = s_ctr0.bia) and (state_d0 = S2A_I2C_ADDR_ACK) then
          s_byte_cnt_rst <= '1';
        end if;

      when S3A_WISHBONE_ADDR_ACK =>
        if (state_d0 = S3_WISHBONE_ADDR) then
          s_bit_cnt_rst <= '1';
        end if;

      when S4_DETECT_OPERATION =>
        if (state_d0 = S3A_WISHBONE_ADDR_ACK) then
          s_bit_cnt_rst  <= '1';
          s_byte_cnt_rst <= '1';
        end if;

      when S5W1_I2C_ADDR =>
        null;

      when S5W1A_I2C_ADDR_ACK =>
        if (state_d0 <= S5W1_I2C_ADDR) then
          s_bit_cnt_rst <= '1';
        end if;

      when S5W2_WRITE_SDA =>
        null;

      when S5W2A_WRITE_SDA_ACK =>
        if (state_d0 = S5W2_WRITE_SDA) then
          s_bit_cnt_rst <= '1';
        end if;

      when S5W0_RESTART =>
        if (state_d0 = S4_DETECT_OPERATION) then
          s_bit_cnt_rst <= '1';
        end if;

      when S5R_READ_SDA =>
        if (s_byte_cnt = s_ctr0.bia) and (state_d0 = S3A_WISHBONE_ADDR_ACK) then
          s_byte_cnt_rst <= '1';
        end if;
        if (state_d0 = S5RA_READ_SDA_ACK) then
          s_bit_cnt_rst <= '1';
        end if;

      when S5RA_READ_SDA_ACK =>
        if (state_d0 = S5R_READ_SDA) then
          s_bit_cnt_rst <= '1';
        end if;

      when S6_WAIT_START_PAUSE =>
        if (state_d0 /= S6_WAIT_START_PAUSE) then
          s_bit_cnt_rst  <= '1';
          s_byte_cnt_rst <= '1';
        end if;

      when S7_PAUSE_DETECT =>
        null;

      when others =>
        null;

    end case;

  end process;

  p_byte_counter_en: process (state, state_d0)
  begin
    s_byte_cnt_en <= '0';

    case state is

      when R0_RESET =>
        null;
         --   if state_d0 /= R0_RESET then
         --   end if;

      when S3A_WISHBONE_ADDR_ACK =>
        if (state_d0 = S3_WISHBONE_ADDR) then
          s_byte_cnt_en <= '1';
        end if;

      when S5RA_READ_SDA_ACK =>
        if (state_d0 = S5R_READ_SDA) then
          s_byte_cnt_en <= '1';
        end if;

      when S5W2A_WRITE_SDA_ACK =>
        if (state_d0 = S5W2_WRITE_SDA) then
          s_byte_cnt_en <= '1';
        end if;

      when others =>
        null;
    end case;
  end process p_byte_counter_en;

  -- Watchdog reset process
  p_watchdog_rst: process(clk_i)
  begin
    if rising_edge(clk_i) then
      s_watchdog_cnt_rst <= '0';
      if rst_i = '1' then
        s_watchdog_cnt_rst <= '1';
      end if;
      if s_watchdog_cnt >= c_WATCHDOG_END_VALUE then
        s_watchdog_cnt_rst <= '1';
      end if;
      if state /= state_d0 then
        s_watchdog_cnt_rst <= '1';
      end if;
    end if;
  end process;


  -- Small process that records the length of a
  -- bit over the I2C interface.
  p_lt_sclp: process(clk_i)
    variable v_count  : std_logic_vector(23 downto 0);
  begin
    if rising_edge(clk_i) then
      if state = R0_RESET then
        s_LT.SCLP <= c_LT_default.SCLP;
      else
        if (state    = S2A_I2C_ADDR_ACK) and (state_d0 = S2_I2C_ADDR) then
          -- Here we do a division bit 8, because the watchdog timer counts for
          -- 8 scl clocks
          v_count   := s_watchdog_cnt_slv(24 - 1 + 3 downto 3);
          s_LT.SCLP <= UNSIGNED(v_count);
        end if;
      end if;
    end if;
  end process p_lt_sclp;

  -- Process to set the next fsm
  p_fsm: process(clk_i)

    procedure check_start_stop is
    begin
      if (s_start_o = '1') then
        state <= S1_START_DETECT;
      elsif (s_pause_o = '1') then
        state <= R0_RESET;
      end if;
    end procedure;

  begin

    if rising_edge(clk_i) then
      if (rst_i = '1') or (s_watchdog_cnt >= c_WATCHDOG_END_VALUE) then
        state <= R0_RESET;
        txsr  <= (others => '0');
      else
        case state is

          when R0_RESET =>
            state <= S0_IDLE;

          when S0_IDLE =>
            if (s_bit_done = '1') then
              if (s_start_o = '1') then
                state <= S1_START_DETECT;
              else
                state <= R0_RESET;
              end if;
            end if;

          when S1_START_DETECT =>
            state <= S2_I2C_ADDR;

          when S2_I2C_ADDR =>
            if (s_bit_done = '1') then
              if (s_bit_cnt = 7) then
                if (s_DRX_slv(6 downto 0) = "0000000") or
                   (s_DRX_slv(6 downto 0) = std_logic_vector(s_CTR0.I2C_ADDR)) then
                  state <= S2A_I2C_ADDR_ACK;
                else
                  state <= S2NA_I2C_ADDR_NACK;
                end if;
              end if;
              check_start_stop;
            end if;

          when S2A_I2C_ADDR_ACK =>
            if (s_bit_done = '1') then
              state <= S3_WISHBONE_ADDR;
            end if;

          when S2NA_I2C_ADDR_NACK =>
            if (s_bit_done = '1') then
              state <= S6_WAIT_START_PAUSE;
            end if;

          when S3_WISHBONE_ADDR =>
            if (s_bit_done = '1') then
              if (s_bit_cnt = 7) then
                state <= S3A_WISHBONE_ADDR_ACK;
              end if;
              check_start_stop;
            end if;

          when S3A_WISHBONE_ADDR_ACK =>
            if (s_bit_done = '1') then
              if (s_byte_cnt = s_ctr0.bia) then
                state <= S4_DETECT_OPERATION;
              else
                state <= S3_WISHBONE_ADDR;
              end if;
            end if;

          when S4_DETECT_OPERATION =>
            if (s_bit_done = '1') then
              if (s_start_o = '1') then
                state <= S5W0_RESTART;
              elsif (s_pause_o = '1') then
                state <= R0_RESET;
              else
                state <= S5R_READ_SDA;
                check_start_stop;
              end if;
            end if;

          when S5R_READ_SDA =>
            if (s_bit_done = '1') then
              if (s_bit_cnt = 7) then
                state <= S5RA_READ_SDA_ACK;
              end if;
              check_start_stop;
            end if;

          when S5RA_READ_SDA_ACK =>
            if (s_bit_done = '1') then
              if (s_byte_cnt < s_ctr0.brd) then
                state <= S5R_READ_SDA;
              else
                state <= S6_WAIT_START_PAUSE;
              end if;
            end if;

          when S5W0_RESTART =>
            state <= S5W1_I2C_ADDR;

          when S5W1_I2C_ADDR =>
            if (s_bit_done = '1') then
              if (s_bit_cnt = 7) then
                if (s_DRX_slv(6 downto 0) = "0000000")  -- ????????????????
                or (s_DRX_slv(6 downto 0) = std_logic_vector(s_CTR0.I2C_ADDR)) then
                  state  <= S5W1A_I2C_ADDR_ACK;
                else
                  state  <= S5W1NA_I2C_ADDR_NACK;
                end if;
              end if;
              check_start_stop;
            end if;

          when S5W1A_I2C_ADDR_ACK =>
            txsr <= f_ch_endian(pf_wb_data_i);
            if s_bit_done = '1' then
              state        <= S5W2_WRITE_SDA;
            end if;

          when S5W1NA_I2C_ADDR_NACK =>
            if s_bit_done = '1' then
              state        <= S6_WAIT_START_PAUSE;
            end if;

          when S5W2_WRITE_SDA =>
            if (s_bit_done = '1') then
              txsr <= txsr(30 downto 0) & '0';
              if (s_bit_cnt = 8) then
                state <= S5W2A_WRITE_SDA_ACK;
              end if;
              --! It can be removed, never reached
              check_start_stop;
            end if;

          when S5W2A_WRITE_SDA_ACK =>
            if (s_bit_done = '1') then
              if (s_rcved_o = '0') then
                if (s_byte_cnt < s_ctr0.bwr) then
                  state <= S5W2_WRITE_SDA;
                else
                  state <= R0_RESET;
                end if;
              else
                state <= S5W3_NACK;
                check_start_stop;
              end if;
            end if;

          when S5W3_NACK =>
            state <= S6_WAIT_START_PAUSE;

          when S6_WAIT_START_PAUSE =>
            if (s_bit_done = '1') then
              if (s_pause_o = '1') then
                state <= S7_PAUSE_DETECT;
              elsif (s_start_o = '1') then
                state <= S1_START_DETECT;
              else
                state <= R0_RESET;
                check_start_stop;
              end if;
            end if;

          when S7_PAUSE_DETECT =>
            state <= R0_RESET;
            check_start_stop;

          when others =>
            state <= R0_RESET;
        end case;
      end if;
    end if;
  end process p_fsm;

end Behavioral;
