--------------------------------------------------------------------------------
-- Company:        CERN, BE-CO-HT CHANGED
-- Engineer:       Carlos Gil Soriano
--
-- Create Date:    18:15:56 11/09/2011
-- Design Name:    A I2C slave with wishbone slave output and interrupt for MCU
-- Module Name:    i2c_slave_top - Behavioral
-- Project Name:   CTDAH
-- Target Devices:
-- Tool versions:
-- Description:
--
-- Dependencies:
--
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
--
----------------------------------------------------------------------------------
library IEEE;
library work;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.i2c_slave_pkg.ALL;
use work.ctdah_pkg.ALL;

entity i2c_slave_top is
  generic 
  (
    g_WB_CLK_PERIOD : TIME := c_WB_CLK_PERIOD  -- Specify in ns 
  ); 
  port 
  (
    sda_oen          : out STD_LOGIC;
    sda_i            : in  STD_LOGIC;
    sda_o            : out STD_LOGIC;
    scl_oen          : out STD_LOGIC;
    scl_i            : in  STD_LOGIC;
    scl_o            : out STD_LOGIC;

    wb_clk_i         : in  STD_LOGIC;
    wb_rst_i         : in  STD_LOGIC;

    wb_master_stb_o  : out STD_LOGIC;
    wb_master_cyc_o  : out STD_LOGIC;
    wb_master_sel_o  : out STD_LOGIC_VECTOR(3  downto 0);
    wb_master_we_o   : out STD_LOGIC;
    wb_master_data_i : in  STD_LOGIC_VECTOR(31 downto 0);
    wb_master_data_o : out STD_LOGIC_VECTOR(31 downto 0);
    wb_master_addr_o : out STD_LOGIC_VECTOR(15  downto 0);
    wb_master_ack_i  : in  STD_LOGIC;
    wb_master_rty_i  : in  STD_LOGIC;
    wb_master_err_i  : in  STD_LOGIC;

    wb_slave_stb_i   : in  STD_LOGIC;
    wb_slave_cyc_i   : in  STD_LOGIC;
    wb_slave_sel_i   : in  STD_LOGIC_VECTOR(3  downto 0);
    wb_slave_we_i    : in  STD_LOGIC;
    wb_slave_data_i  : in  STD_LOGIC_VECTOR(31 downto 0);
    wb_slave_data_o  : out STD_LOGIC_VECTOR(31 downto 0);
    wb_slave_addr_i  : in  STD_LOGIC_VECTOR(3  downto 0);
    wb_slave_ack_o   : out STD_LOGIC;
    wb_slave_rty_o   : out STD_LOGIC;
    wb_slave_err_o   : out STD_LOGIC;

    pf_wb_addr_o     : out STD_LOGIC;
    rd_done_o        : out STD_LOGIC;
    wr_done_o        : out STD_LOGIC;
    i2c_addr_i       : in  STD_LOGIC_VECTOR(6  downto 0)
  );
end i2c_slave_top;

architecture Behavioral of i2c_slave_top is

  signal ctr0          : STD_LOGIC_VECTOR(r_CTR0'a_length - 1 downto 0);
  signal lt            : STD_LOGIC_VECTOR(r_LT'a_length   - 1 downto 0);

  signal drxa        : STD_LOGIC_VECTOR(r_DRX'a_length - 1 downto 0);
  signal drxb        : STD_LOGIC_VECTOR(r_DRX'a_length - 1 downto 0);

  signal pf_wb_addr  : STD_LOGIC;
  signal pf_wb_data  : STD_LOGIC_VECTOR(31 downto 0);
  signal rd_done     : STD_LOGIC;
  signal wr_done     : STD_LOGIC;

  signal s_clk_i2c     : STD_LOGIC;
  signal rst_i2c       : STD_LOGIC;
  signal reset_extender: STD_LOGIC_VECTOR(2**c_RST_EXTENSOR - 1 downto 0) := (others => '1');

begin

  pf_wb_addr_o <= pf_wb_addr;
  rd_done_o    <= rd_done;
  wr_done_o    <= wr_done;

  cmp_i2c_slave_core: i2c_slave_core
  port map
  (
    clk_i        => wb_clk_i,
    rst_i        => wb_rst_i,

    sda_oen      => sda_oen,
    sda_i        => sda_i,
    sda_o        => sda_o,
    scl_oen      => scl_oen,
    scl_i        => scl_i,
    scl_o        => scl_o,

    CTR0_i       => ctr0,
    LT_o         => lt,
    DRXA_o       => drxa,
    DRXB_o       => drxb,

    pf_wb_addr_o => pf_wb_addr,
    pf_wb_data_i => pf_wb_data,
    rd_done_o    => rd_done,
    wr_done_o    => wr_done
  );

  cmp_i2c_regs: i2c_regs
  port map
  (
    pf_wb_addr_i     => pf_wb_addr,
    pf_wb_data_o     => pf_wb_data,
    rd_done_i        => rd_done,
    wr_done_i        => wr_done,

    wb_rst_i         => wb_rst_i,
    wb_clk_i         => wb_clk_i,

    wb_master_we_o   => wb_master_we_o,
    wb_master_stb_o  => wb_master_stb_o,
    wb_master_cyc_o  => wb_master_cyc_o,
    wb_master_sel_o  => wb_master_sel_o,
    wb_master_data_i => wb_master_data_i,
    wb_master_data_o => wb_master_data_o,
    wb_master_addr_o => wb_master_addr_o,
    wb_master_ack_i  => wb_master_ack_i,
    wb_master_rty_i  => wb_master_rty_i,
    wb_master_err_i  => wb_master_err_i,

    wb_slave_we_i    => wb_slave_we_i,
    wb_slave_stb_i   => wb_slave_stb_i,
    wb_slave_cyc_i   => wb_slave_cyc_i,
    wb_slave_sel_i   => wb_slave_sel_i,
    wb_slave_data_i  => wb_slave_data_i,
    wb_slave_data_o  => wb_slave_data_o,
    wb_slave_addr_i  => wb_slave_addr_i,
    wb_slave_ack_o   => wb_slave_ack_o,
    wb_slave_rty_o   => wb_slave_rty_o,
    wb_slave_err_o   => wb_slave_err_o,

    CTR0_o           => ctr0,
    LT_i             => lt,
    DRXA_i           => drxa,
    DRXB_i           => drxb,
    i2c_addr_i       => i2c_addr_i
  );

  rst_i2c <= reset_extender(2**c_RST_EXTENSOR - 1);

   --! A shift with reset, consumes just a few SLICEX in Spartan6.
  p_rst_extender : process(wb_clk_i)
  begin
    if rising_edge(wb_clk_i) then
      if wb_rst_i = '1' then
        reset_extender <= (others => '1');
      else
        reset_extender(0) <= '0';
        for i in 1 to 2**c_RST_EXTENSOR -1 loop
          reset_extender(i) <= reset_extender(i-1);
        end loop;
      end if;
    end if;
  end process;

end Behavioral;
