library IEEE;
library work;

use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.ALL;
use work.i2c_slave_pkg.ALL;

package i2c_tb_pkg is

   --! Clock period definitions
   constant c_WB_CLK_PERIOD   : time := 50 ns;     -- @ 20 MHz
   constant c_SCL_I_PERIOD    : time := 10000 ns;  -- @ 400 KHz

   --! Will be used as the LSB of the I2C
   constant c_VME_SLOT        : UNSIGNED(4 downto 0)  := "11110";

   --! Length of some buses
   constant c_WR_DATA_LENGTH : NATURAL := 32;
   constant c_RD_DATA_LENGTH : NATURAL := 32;
   constant c_WB_ADDR_LENGTH : NATURAL := 16;

   component i2c_slave_top is
   port (sda_oen          : out  STD_LOGIC;
         sda_i            : in   STD_LOGIC;
         sda_o            : out  STD_LOGIC;
         scl_oen          : out  STD_LOGIC;
         scl_i            : in   STD_LOGIC;
         scl_o            : out  STD_LOGIC;

         wb_clk           : in   STD_LOGIC;
         wb_rst_i         : in   STD_LOGIC;

         wb_master_stb_o  : out STD_LOGIC;
         wb_master_cyc_o  : out STD_LOGIC;
         wb_master_sel_o  : out STD_LOGIC_VECTOR(3  downto 0);
         wb_master_we_o   : out STD_LOGIC;
         wb_master_data_i : in  STD_LOGIC_VECTOR(31 downto 0);
         wb_master_data_o : out STD_LOGIC_VECTOR(31 downto 0);
         wb_master_addr_o : out STD_LOGIC_VECTOR(15  downto 0);
         wb_master_ack_i  : in  STD_LOGIC;
         wb_master_rty_i  : in  STD_LOGIC;
         wb_master_err_i  : in  STD_LOGIC;

         wb_slave_stb_i   : in  STD_LOGIC;
         wb_slave_cyc_i   : in  STD_LOGIC;
         wb_slave_sel_i   : in  STD_LOGIC_VECTOR(3  downto 0);
         wb_slave_we_i    : in  STD_LOGIC;
         wb_slave_data_i  : in  STD_LOGIC_VECTOR(31 downto 0);
         wb_slave_data_o  : out STD_LOGIC_VECTOR(31 downto 0);
         wb_slave_addr_i  : in  STD_LOGIC_VECTOR(3  downto 0);
         wb_slave_ack_o   : out STD_LOGIC;
         wb_slave_rty_o   : out STD_LOGIC;
         wb_slave_err_o   : out STD_LOGIC;

         pf_wb_addr_o     : out STD_LOGIC;
         rd_done_o        : out STD_LOGIC;
         wr_done_o        : out STD_LOGIC;
         i2c_addr_i       : in  STD_LOGIC_VECTOR(6  downto 0));
   end component;


   component i2c_master_driver is
   generic(g_WR_DATA_LENGTH : NATURAL := c_WR_DATA_LENGTH;
           g_RD_DATA_LENGTH : NATURAL := c_RD_DATA_LENGTH;
           g_WB_ADDR_LENGTH : NATURAL := c_WB_ADDR_LENGTH;
           g_SCL_PERIOD     : TIME    := c_SCL_I_PERIOD;
           g_LOG_PATH       : STRING  := "../test/log/i2c_master_driver.txt");
   port(tb_clk          : in  STD_LOGIC;
        rst_n           : in  std_logic;

        sda_master_i    : in  STD_LOGIC;
        sda_master_o    : out STD_LOGIC;
        scl_master_o    : out STD_LOGIC;

        i2c_addr_op_i   : in  STD_LOGIC_VECTOR(7 downto 0);
        wishbone_addr_i : in  STD_LOGIC_VECTOR(g_WB_ADDR_LENGTH - 1 downto 0);
        wr_data_i       : in  STD_LOGIC_VECTOR(g_WR_DATA_LENGTH - 1 downto 0);
        rd_data_o       : out STD_LOGIC_VECTOR(g_RD_DATA_LENGTH - 1 downto 0);

        start_i         : in  STD_LOGIC;
        start_done_o    : out STD_LOGIC;
        pause_i         : in  STD_LOGIC;
        pause_done_o    : out STD_LOGIC;
        write_i         : in  STD_LOGIC;
        write_done_o    : out STD_LOGIC;
        read_i          : in  STD_LOGIC;
        read_done_o     : out STD_LOGIC);
   end component;

   type I2C_master_driver_ctrl is
      record
        START    : STD_LOGIC;
        PAUSE    : STD_LOGIC;
        WRITE    : STD_LOGIC;
        READ     : STD_LOGIC;
      end record;

   constant c_I2C_master_driver_ctrl_default  : I2C_master_driver_ctrl
                                                :=(START   => '0',
                                                   PAUSE   => '0',
                                                   WRITE   => '0',
                                                   READ    => '0');

end i2c_tb_pkg;

package body i2c_tb_pkg is
end i2c_tb_pkg;
