library IEEE;
library work;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.multiboot_pkg.ALL;

entity multiboot_regs is
   generic(g_MBA_addr        : UNSIGNED(23 downto 0);
           g_GBA_addr        : UNSIGNED(23 downto 0);
           g_READ_SPI_OPCODE : STD_LOGIC_VECTOR(7 downto 0));
   port(wb_rst_i   : in  STD_LOGIC;
        wb_clk     : in  STD_LOGIC;
        wb_we_i    : in  STD_LOGIC;
        wb_stb_i   : in  STD_LOGIC;
        wb_cyc_i   : in  STD_LOGIC;
        wb_sel_i   : in  STD_LOGIC_VECTOR (3  downto 0);
        wb_data_i  : in  STD_LOGIC_VECTOR (31 downto 0);
        wb_data_o  : out STD_LOGIC_VECTOR (31 downto 0);
        wb_addr_i  : in  STD_LOGIC_VECTOR (3  downto 0);
        wb_ack_o   : out STD_LOGIC;
        wb_rty_o   : out STD_LOGIC;
        wb_err_o   : out STD_LOGIC;

        CTR0_o     : out r_CTR0;
        CTR1_i     : in  r_CTR1;
        STAT_i     : in  r_STAT;
        MBA_o      : out r_BAR;
        GBA_o      : out r_BAR;
        MBA_ICAP_i : in  STD_LOGIC_VECTOR (23 downto 0);
        GBA_ICAP_i : in  STD_LOGIC_VECTOR (23 downto 0));
end multiboot_regs;

architecture Behavioral of multiboot_regs is

   signal s_CTR0      : r_CTR0 := c_CTR0_default;
   signal s_CTR1      : r_CTR1;
   signal s_CTR1_d0   : r_CTR1;
   signal s_STAT      : r_STAT;

   --! SPI_OPCODE can be found in xtp059.pdf slide 25.
   --! It corresponds to the instruction it should be performed in
   --! the SPI interface, i.e. a read instruction, starting at a 
   --! given point.
   --! This should be checked agains the vendor instruction set for
   --! the SPI device.
   signal s_MBA       : r_BAR := f_BAR(  g_READ_SPI_OPCODE
                                       & STD_LOGIC_VECTOR(g_MBA_addr));
   signal s_GBA       : r_BAR := f_BAR(  g_READ_SPI_OPCODE
                                       & STD_LOGIC_VECTOR(g_GBA_addr));

   signal s_MBA_ICAP  : STD_LOGIC_VECTOR(23 downto 0);
   signal s_GBA_ICAP  : STD_LOGIC_VECTOR(23 downto 0);

   signal s_wb_data_i : STD_LOGIC_VECTOR(31 downto 0) := (others => '0');
   signal s_wb_data_o : STD_LOGIC_VECTOR(31 downto 0);
   signal s_wb_ack    : STD_LOGIC := '0';
   signal s_wb_rty    : STD_LOGIC := '0';
   signal s_wb_err    : STD_LOGIC := '0';

begin

   CTR0_o     <= s_CTR0;
   s_CTR1     <= CTR1_i;
   s_STAT     <= STAT_i;

   MBA_o      <= s_MBA;
   GBA_o      <= s_GBA;
   s_MBA_ICAP <= MBA_ICAP_i;
   s_GBA_ICAP <= GBA_ICAP_i;


   process (wb_sel_i,
            wb_data_i,
            s_wb_data_o)
   begin
      for i in 0 to 3 loop
         if wb_sel_i(i) = '1' then
            s_wb_data_i(8*(i+1) - 1  downto  8*i) <=
                                     wb_data_i(8*(i+1) - 1    downto  8*i);
            wb_data_o(8*(i+1) - 1    downto  8*i) <=
                                     s_wb_data_o(8*(i+1) - 1  downto  8*i);
         else
            s_wb_data_i(8*(i+1) - 1  downto  8*i) <= (others => '0');
            wb_data_o(8*(i+1) - 1    downto  8*i) <= (others => '0');
         end if;
      end loop;
   end process;

   wb_ack_o   <= s_wb_ack;
   wb_rty_o   <= s_wb_rty;
   wb_err_o   <= s_wb_err;

   process (wb_clk)
      variable v_wb_addr  : UNSIGNED(3 downto 0) := to_unsigned(0, 4);
   begin
      if rising_edge(wb_clk) then
         v_wb_addr := UNSIGNED(wb_addr_i);
         if wb_rst_i = '1' then
            s_CTR0       <= c_CTR0_default;
            s_wb_data_o  <= (others => '0');
            s_wb_ack     <= '0';
            s_wb_rty     <= '0';
            s_wb_err     <= '0';
         else
            s_wb_ack     <= '0';
            s_wb_rty     <= '0';
            s_wb_err     <= '0';
            s_CTR1_d0    <= s_CTR1;

            if     s_CTR1.FSM_RUN    = '0'
               and s_CTR1_d0.FSM_RUN = '1' then
               --! When we detect that the operation has been already
               --! performed, we release the lock.
               s_CTR0.PEN <= '0';
            end if;

            if    (wb_stb_i = '1' and wb_cyc_i = '1')
                  and s_wb_ack = '0'
                  and s_wb_rty = '0'
                  and s_wb_err = '0' then
               if s_CTR1.FSM_RUN = '1' then
                  s_wb_err <= '1';
               else
                  case wb_we_i is
                     when '0'    =>
                        case v_wb_addr is
                           when CTR0_addr       =>
                              s_wb_data_o (31 downto 8) <= (others => '0');
                              s_wb_data_o ( 7 downto 0)
                                          <= f_STD_LOGIC_VECTOR(s_CTR0);
                              s_wb_ack  <= '1';
                           when CTR1_addr       =>
                              s_wb_data_o (31 downto 8) <= (others => '0');
                              s_wb_data_o ( 7 downto 0)
                                          <= f_STD_LOGIC_VECTOR(s_CTR1);
                              s_wb_ack  <= '1';
                           when STAT_addr       =>
                              s_wb_data_o (31 downto 16) <= (others => '0');
                              s_wb_data_o (15 downto  0)
                                          <= f_STD_LOGIC_VECTOR(s_STAT);
                              s_wb_ack  <= '1';
                           when c_MBA_addr      =>
                              s_wb_data_o <= f_STD_LOGIC_VECTOR(s_MBA);
                              s_wb_ack  <= '1';
                           when c_GBA_addr      =>
                              s_wb_data_o <= f_STD_LOGIC_VECTOR(s_GBA);
                              s_wb_ack  <= '1';
                           when c_MBA_ICAP_addr =>
                              s_wb_data_o (31 downto 24) <= (others => '0');
                              s_wb_data_o (23 downto  0) <= s_MBA_ICAP;
                              s_wb_ack  <= '1';
                           when c_GBA_ICAP_addr =>
                              s_wb_data_o (31 downto 24) <= (others => '0');
                              s_wb_data_o (23 downto  0) <= s_GBA_ICAP;
                              s_wb_ack  <= '1';
                           when others =>
                              s_wb_err  <= '1';
                        end case;
                     when others =>
                        case v_wb_addr is
                           when CTR0_addr       =>
                              s_CTR0    <= f_CTR0(s_wb_data_i(7 downto 0));
                              s_wb_ack  <= '1';
                           when c_MBA_addr      =>
                              s_MBA     <= f_BAR(s_wb_data_i);
                              s_wb_ack  <= '1';
                           when c_GBA_addr      =>
                              s_GBA     <= f_BAR(s_wb_data_i);
                              s_wb_ack  <= '1';
                           when others =>
                              s_wb_err  <= '1';
                        end case;
                  end case;
               end if;
            end if;
         end if;
      end if;
   end process;

end Behavioral;
