#==========================================================================
# Beagle Interface Library
#--------------------------------------------------------------------------
# Copyright (c) 2004-2008 Total Phase, Inc.
# All rights reserved.
# www.totalphase.com
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# - Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# - Neither the name of Total Phase, Inc. nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#--------------------------------------------------------------------------
# To access Total Phase Beagle devices through the API:
#
# 1) Use one of the following shared objects:
#      beagle.so        --  Linux shared object
#          or
#      beagle.dll       --  Windows dynamic link library
#
# 2) Along with one of the following language modules:
#      beagle.c/h       --  C/C++ API header file and interface module
#      beagle_py.py     --  Python API
#      beagle.bas       --  Visual Basic 6 API
#      beagle.cs        --  C# .NET source
#      beagle_net.dll   --  Compiled .NET binding
#==========================================================================


#==========================================================================
# VERSION
#==========================================================================
BG_API_VERSION    = 0x0304   # v3.04
BG_REQ_SW_VERSION = 0x0307   # v3.07

import os
import sys
try:
    import beagle as api
except ImportError, ex1:
    import imp, platform
    ext = platform.system() == 'Windows' and '.dll' or '.so'
    try:
        api = imp.load_dynamic('beagle', 'beagle' + ext)
    except ImportError, ex2:
        msg  = 'Error importing beagle%s\n' % ext
        msg += '  Architecture of beagle%s may be wrong\n' % ext
        msg += '%s\n%s' % (ex1, ex2)
        raise ImportError(msg)

BG_SW_VERSION      = api.py_version() & 0xffff
BG_REQ_API_VERSION = (api.py_version() >> 16) & 0xffff
BG_LIBRARY_LOADED  = \
    ((BG_SW_VERSION >= BG_REQ_SW_VERSION) and \
     (BG_API_VERSION >= BG_REQ_API_VERSION))

from array import array, ArrayType
import struct


#==========================================================================
# HELPER FUNCTIONS
#==========================================================================
def array_u08 (n):  return array('B', '\0'*n)
def array_u16 (n):  return array('H', '\0\0'*n)
def array_u32 (n):  return array('I', '\0\0\0\0'*n)
def array_u64 (n):  return array('K', '\0\0\0\0\0\0\0\0'*n)
def array_s08 (n):  return array('b', '\0'*n)
def array_s16 (n):  return array('h', '\0\0'*n)
def array_s32 (n):  return array('i', '\0\0\0\0'*n)
def array_s64 (n):  return array('L', '\0\0\0\0\0\0\0\0'*n)


#==========================================================================
# STATUS CODES
#==========================================================================
# All API functions return an integer which is the result of the
# transaction, or a status code if negative.  The status codes are
# defined as follows:
# enum BeagleStatus
# General codes (0 to -99)
BG_OK                      =    0
BG_UNABLE_TO_LOAD_LIBRARY  =   -1
BG_UNABLE_TO_LOAD_DRIVER   =   -2
BG_UNABLE_TO_LOAD_FUNCTION =   -3
BG_INCOMPATIBLE_LIBRARY    =   -4
BG_INCOMPATIBLE_DEVICE     =   -5
BG_INCOMPATIBLE_DRIVER     =   -6
BG_COMMUNICATION_ERROR     =   -7
BG_UNABLE_TO_OPEN          =   -8
BG_UNABLE_TO_CLOSE         =   -9
BG_INVALID_HANDLE          =  -10
BG_CONFIG_ERROR            =  -11
BG_UNKNOWN_PROTOCOL        =  -12
BG_STILL_ACTIVE            =  -13
BG_FUNCTION_NOT_AVAILABLE  =  -14

# COMMTEST codes (-100 to -199)
BG_COMMTEST_NOT_AVAILABLE  = -100
BG_COMMTEST_NOT_ENABLED    = -101

# I2C codes (-200 to -299)
BG_I2C_NOT_AVAILABLE       = -200
BG_I2C_NOT_ENABLED         = -201

# SPI codes (-300 to -399)
BG_SPI_NOT_AVAILABLE       = -300
BG_SPI_NOT_ENABLED         = -301

# USB codes (-400 to -499)
BG_USB_NOT_AVAILABLE       = -400
BG_USB_NOT_ENABLED         = -401

# MDIO codes (-500 to -599)
BG_MDIO_NOT_AVAILABLE      = -500
BG_MDIO_NOT_ENABLED        = -501
BG_MDIO_BAD_TURNAROUND     = -502


#==========================================================================
# GENERAL TYPE DEFINITIONS
#==========================================================================
# Beagle handle type definition
# typedef Beagle => integer

# Beagle version matrix.
#
# This matrix describes the various version dependencies
# of Beagle components.  It can be used to determine
# which component caused an incompatibility error.
#
# All version numbers are of the format:
#   (major << 8) | minor
#
# ex. v1.20 would be encoded as:  0x0114
class BeagleVersion:
    def __init__ (self):
        # Software, firmware, and hardware versions.
        self.software        = 0
        self.firmware        = 0
        self.hardware        = 0

        # Hardware revisions that are compatible with this software version.
        # The top 16 bits gives the maximum accepted hw revision.
        # The lower 16 bits gives the minimum accepted hw revision.
        self.hw_revs_for_sw  = 0

        # Firmware revisions that are compatible with this software version.
        # The top 16 bits gives the maximum accepted fw revision.
        # The lower 16 bits gives the minimum accepted fw revision.
        self.fw_revs_for_sw  = 0

        # Driver revisions that are compatible with this software version.
        # The top 16 bits gives the maximum accepted driver revision.
        # The lower 16 bits gives the minimum accepted driver revision.
        # This version checking is currently only pertinent for WIN32
        # platforms.
        self.drv_revs_for_sw = 0

        # Software requires that the API interface must be >= this version.
        self.api_req_by_sw   = 0


#==========================================================================
# GENERAL API
#==========================================================================
# Get a list of ports to which Beagle devices are attached.
#
# num_devices = maximum number of elements to return
# devices     = array into which the port numbers are returned
#
# Each element of the array is written with the port number.
# Devices that are in-use are ORed with BG_PORT_NOT_FREE
# (0x8000).
#
# ex.  devices are attached to ports 0, 1, 2
#      ports 0 and 2 are available, and port 1 is in-use.
#      array => 0x0000, 0x8001, 0x0002
#
# If the array is NULL, it is not filled with any values.
# If there are more devices than the array size, only the
# first nmemb port numbers will be written into the array.
#
# Returns the number of devices found, regardless of the
# array size.
BG_PORT_NOT_FREE = 0x8000
def bg_find_devices (devices):
    """usage: (int return, u16[] devices) = bg_find_devices(u16[] devices)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function.

    Additionally, for arrays that are filled by the API function, an
    integer can be passed in place of the array argument and the API
    will automatically create an array of that length.  All output
    arrays, whether passed in or generated, are passed back in the
    returned tuple."""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # devices pre-processing
    __devices = isinstance(devices, int)
    if __devices:
        (devices, num_devices) = (array_u16(devices), devices)
    else:
        (devices, num_devices) = isinstance(devices, ArrayType) and (devices, len(devices)) or (devices[0], min(len(devices[0]), int(devices[1])))
        if devices.typecode != 'H':
            raise TypeError("type for 'devices' must be array('H')")
    # Call API function
    (_ret_) = api.py_bg_find_devices(num_devices, devices)
    # devices post-processing
    if __devices: del devices[max(0, min(_ret_, len(devices))):]
    return (_ret_, devices)


# Get a list of ports to which Beagle devices are attached
#
# This function is the same as bg_find_devices() except that
# it returns the unique IDs of each Beagle device.  The IDs
# are guaranteed to be non-zero if valid.
#
# The IDs are the unsigned integer representation of the 10-digit
# serial numbers.
def bg_find_devices_ext (devices, unique_ids):
    """usage: (int return, u16[] devices, u32[] unique_ids) = bg_find_devices_ext(u16[] devices, u32[] unique_ids)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function.

    Additionally, for arrays that are filled by the API function, an
    integer can be passed in place of the array argument and the API
    will automatically create an array of that length.  All output
    arrays, whether passed in or generated, are passed back in the
    returned tuple."""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # devices pre-processing
    __devices = isinstance(devices, int)
    if __devices:
        (devices, num_devices) = (array_u16(devices), devices)
    else:
        (devices, num_devices) = isinstance(devices, ArrayType) and (devices, len(devices)) or (devices[0], min(len(devices[0]), int(devices[1])))
        if devices.typecode != 'H':
            raise TypeError("type for 'devices' must be array('H')")
    # unique_ids pre-processing
    __unique_ids = isinstance(unique_ids, int)
    if __unique_ids:
        (unique_ids, num_ids) = (array_u32(unique_ids), unique_ids)
    else:
        (unique_ids, num_ids) = isinstance(unique_ids, ArrayType) and (unique_ids, len(unique_ids)) or (unique_ids[0], min(len(unique_ids[0]), int(unique_ids[1])))
        if unique_ids.typecode != 'I':
            raise TypeError("type for 'unique_ids' must be array('I')")
    # Call API function
    (_ret_) = api.py_bg_find_devices_ext(num_devices, num_ids, devices, unique_ids)
    # devices post-processing
    if __devices: del devices[max(0, min(_ret_, len(devices))):]
    # unique_ids post-processing
    if __unique_ids: del unique_ids[max(0, min(_ret_, len(unique_ids))):]
    return (_ret_, devices, unique_ids)


# Open the Beagle port.
#
# The port number is a zero-indexed integer.
#
# The port number is the same as that obtained from the
# bg_find_devices() function above.
#
# Returns an Beagle handle, which is guaranteed to be
# greater than zero if it is valid.
#
# This function is recommended for use in simple applications
# where extended information is not required.  For more complex
# applications, the use of bg_open_ext() is recommended.
def bg_open (port_number):
    """usage: Beagle return = bg_open(int port_number)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_open(port_number)


# Open the Beagle port, returning extended information
# in the supplied structure.  Behavior is otherwise identical
# to bg_open() above.  If 0 is passed as the pointer to the
# structure, this function is exactly equivalent to bg_open().
#
# The structure is zeroed before the open is attempted.
# It is filled with whatever information is available.
#
# For example, if the hardware version is not filled, then
# the device could not be queried for its version number.
#
# This function is recommended for use in complex applications
# where extended information is required.  For more simple
# applications, the use of bg_open() is recommended.
class BeagleExt:
    def __init__ (self):
        # Version matrix
        self.version  = BeagleVersion()

        # Features of this device.
        self.features = 0

def bg_open_ext (port_number):
    """usage: (Beagle return, BeagleExt bg_ext) = bg_open_ext(int port_number)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    (_ret_, c_bg_ext) = api.py_bg_open_ext(port_number)
    # bg_ext post-processing
    bg_ext = BeagleExt()
    (bg_ext.version.software, bg_ext.version.firmware, bg_ext.version.hardware, bg_ext.version.hw_revs_for_sw, bg_ext.version.fw_revs_for_sw, bg_ext.version.drv_revs_for_sw, bg_ext.version.api_req_by_sw, bg_ext.features) = c_bg_ext
    return (_ret_, bg_ext)


# Close the Beagle port.
def bg_close (beagle):
    """usage: int return = bg_close(Beagle beagle)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_close(beagle)


# Return the port for this Beagle handle.
#
# The port number is a zero-indexed integer.
def bg_port (beagle):
    """usage: int return = bg_port(Beagle beagle)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_port(beagle)


# Return the device features as a bit-mask of values, or
# an error code if the handle is not valid.
BG_FEATURE_NONE = 0x00000000
BG_FEATURE_I2C = 0x00000001
BG_FEATURE_SPI = 0x00000002
BG_FEATURE_USB = 0x00000004
BG_FEATURE_MDIO = 0x00000008
BG_FEATURE_USB_HS = 0x00000010
def bg_features (beagle):
    """usage: int return = bg_features(Beagle beagle)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_features(beagle)


def bg_unique_id_to_features (unique_id):
    """usage: int return = bg_unique_id_to_features(u32 unique_id)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_unique_id_to_features(unique_id)


# Return the unique ID for this Beagle adapter.
# IDs are guaranteed to be non-zero if valid.
# The ID is the unsigned integer representation of the
# 10-digit serial number.
def bg_unique_id (beagle):
    """usage: u32 return = bg_unique_id(Beagle beagle)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_unique_id(beagle)


# Return the status string for the given status code.
# If the code is not valid or the library function cannot
# be loaded, return a NULL string.
def bg_status_string (status):
    """usage: str return = bg_status_string(int status)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_status_string(status)


# Return the version matrix for the device attached to the
# given handle.  If the handle is 0 or invalid, only the
# software and required api versions are set.
def bg_version (beagle):
    """usage: (int return, BeagleVersion version) = bg_version(Beagle beagle)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    (_ret_, c_version) = api.py_bg_version(beagle)
    # version post-processing
    version = BeagleVersion()
    (version.software, version.firmware, version.hardware, version.hw_revs_for_sw, version.fw_revs_for_sw, version.drv_revs_for_sw, version.api_req_by_sw) = c_version
    return (_ret_, version)


# Set the capture latency to the specified number of milliseconds.
# This number determines the minimum time that a read call will
# block if there is no available data.  Lower times result in
# faster turnaround at the expense of reduced buffering.  Setting
# this parameter too low can cause packets to be dropped.
def bg_latency (beagle, milliseconds):
    """usage: int return = bg_latency(Beagle beagle, u32 milliseconds)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_latency(beagle, milliseconds)


# Set the capture timeout to the specified number of milliseconds.
# If any read call has a longer idle than this value, that call
# will return with a count of 0 bytes.
def bg_timeout (beagle, milliseconds):
    """usage: int return = bg_timeout(Beagle beagle, u32 milliseconds)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_timeout(beagle, milliseconds)


# Sleep for the specified number of milliseconds
# Accuracy depends on the operating system scheduler
# Returns the number of milliseconds slept
def bg_sleep_ms (milliseconds):
    """usage: u32 return = bg_sleep_ms(u32 milliseconds)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_sleep_ms(milliseconds)


# Configure the target power pin.
BG_TARGET_POWER_OFF = 0x00
BG_TARGET_POWER_ON = 0x01
BG_TARGET_POWER_QUERY = 0x80
def bg_target_power (beagle, power_flag):
    """usage: int return = bg_target_power(Beagle beagle, u08 power_flag)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_target_power(beagle, power_flag)


BG_HOST_IFCE_FULL_SPEED = 0x00
BG_HOST_IFCE_HIGH_SPEED = 0x01
def bg_host_ifce_speed (beagle):
    """usage: int return = bg_host_ifce_speed(Beagle beagle)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_host_ifce_speed(beagle)


# Returns the device address that the beagle is attached to.
def bg_dev_addr (beagle):
    """usage: int return = bg_dev_addr(Beagle beagle)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_dev_addr(beagle)



#==========================================================================
# BUFFERING API
#==========================================================================
# Set the amount of buffering that is to be allocated on the PC.
# Pass zero to num_bytes to query the existing buffer size.
def bg_host_buffer_size (beagle, num_bytes):
    """usage: int return = bg_host_buffer_size(Beagle beagle, u32 num_bytes)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_host_buffer_size(beagle, num_bytes)


# Query the amount of buffering that is unused and free for buffering.
def bg_host_buffer_free (beagle):
    """usage: int return = bg_host_buffer_free(Beagle beagle)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_host_buffer_free(beagle)


# Query the amount of buffering that is used and no longer available.
def bg_host_buffer_used (beagle):
    """usage: int return = bg_host_buffer_used(Beagle beagle)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_host_buffer_used(beagle)


# Benchmark the speed of the host to Beagle interface
def bg_commtest (beagle, num_samples, delay_count):
    """usage: int return = bg_commtest(Beagle beagle, int num_samples, int delay_count)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_commtest(beagle, num_samples, delay_count)



#==========================================================================
# MONITORING API
#==========================================================================
# Protocol codes
# enum BeagleProtocol
BG_PROTOCOL_NONE     = 0
BG_PROTOCOL_COMMTEST = 1
BG_PROTOCOL_USB      = 2
BG_PROTOCOL_I2C      = 3
BG_PROTOCOL_SPI      = 4
BG_PROTOCOL_MDIO     = 5

# Common Beagle read status codes
BG_READ_OK = 0x00000000
BG_READ_TIMEOUT = 0x00000100
BG_READ_ERR_MIDDLE_OF_PACKET = 0x00000200
BG_READ_ERR_SHORT_BUFFER = 0x00000400
BG_READ_ERR_PARTIAL_LAST_BYTE = 0x00000800
BG_READ_ERR_PARTIAL_LAST_BYTE_MASK = 0x0000000f
BG_READ_ERR_UNEXPECTED = 0x00001000
# Enable the Beagle monitor
def bg_enable (beagle, protocol):
    """usage: int return = bg_enable(Beagle beagle, BeagleProtocol protocol)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_enable(beagle, protocol)


# Disable the Beagle monitor
def bg_disable (beagle):
    """usage: int return = bg_disable(Beagle beagle)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_disable(beagle)


# Set the sample rate in kilohertz.
def bg_samplerate (beagle, samplerate_khz):
    """usage: int return = bg_samplerate(Beagle beagle, int samplerate_khz)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_samplerate(beagle, samplerate_khz)


# Get the number of bits for the given number of bytes in the
# given protocol.
# Use this to determine how large a bit_timing array to allocate
# for bg_*_read_bit_timing functions.
def bg_bit_timing_size (protocol, num_data_bytes):
    """usage: int return = bg_bit_timing_size(BeagleProtocol protocol, int num_data_bytes)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_bit_timing_size(protocol, num_data_bytes)



#==========================================================================
# I2C API
#==========================================================================
# Configure the I2C pullup resistors.
BG_I2C_PULLUP_OFF = 0x00
BG_I2C_PULLUP_ON = 0x01
BG_I2C_PULLUP_QUERY = 0x80
def bg_i2c_pullup (beagle, pullup_flag):
    """usage: int return = bg_i2c_pullup(Beagle beagle, u08 pullup_flag)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_i2c_pullup(beagle, pullup_flag)


BG_I2C_MONITOR_DATA = 0x00ff
BG_I2C_MONITOR_NACK = 0x0100
BG_READ_I2C_NO_STOP = 0x00010000
def bg_i2c_read (beagle, data_in):
    """usage: (int return, u32 status, u64 time_sop, u64 time_duration, u32 time_dataoffset, u16[] data_in) = bg_i2c_read(Beagle beagle, u16[] data_in)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function.

    Additionally, for arrays that are filled by the API function, an
    integer can be passed in place of the array argument and the API
    will automatically create an array of that length.  All output
    arrays, whether passed in or generated, are passed back in the
    returned tuple."""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # data_in pre-processing
    __data_in = isinstance(data_in, int)
    if __data_in:
        (data_in, max_bytes) = (array_u16(data_in), data_in)
    else:
        (data_in, max_bytes) = isinstance(data_in, ArrayType) and (data_in, len(data_in)) or (data_in[0], min(len(data_in[0]), int(data_in[1])))
        if data_in.typecode != 'H':
            raise TypeError("type for 'data_in' must be array('H')")
    # Call API function
    (_ret_, status, time_sop, time_duration, time_dataoffset) = api.py_bg_i2c_read(beagle, max_bytes, data_in)
    # data_in post-processing
    if __data_in: del data_in[max(0, min(_ret_, len(data_in))):]
    return (_ret_, status, time_sop, time_duration, time_dataoffset, data_in)


def bg_i2c_read_data_timing (beagle, data_in, data_timing):
    """usage: (int return, u32 status, u64 time_sop, u64 time_duration, u32 time_dataoffset, u16[] data_in, u32[] data_timing) = bg_i2c_read_data_timing(Beagle beagle, u16[] data_in, u32[] data_timing)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function.

    Additionally, for arrays that are filled by the API function, an
    integer can be passed in place of the array argument and the API
    will automatically create an array of that length.  All output
    arrays, whether passed in or generated, are passed back in the
    returned tuple."""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # data_in pre-processing
    __data_in = isinstance(data_in, int)
    if __data_in:
        (data_in, max_bytes) = (array_u16(data_in), data_in)
    else:
        (data_in, max_bytes) = isinstance(data_in, ArrayType) and (data_in, len(data_in)) or (data_in[0], min(len(data_in[0]), int(data_in[1])))
        if data_in.typecode != 'H':
            raise TypeError("type for 'data_in' must be array('H')")
    # data_timing pre-processing
    __data_timing = isinstance(data_timing, int)
    if __data_timing:
        (data_timing, max_timing) = (array_u32(data_timing), data_timing)
    else:
        (data_timing, max_timing) = isinstance(data_timing, ArrayType) and (data_timing, len(data_timing)) or (data_timing[0], min(len(data_timing[0]), int(data_timing[1])))
        if data_timing.typecode != 'I':
            raise TypeError("type for 'data_timing' must be array('I')")
    # Call API function
    (_ret_, status, time_sop, time_duration, time_dataoffset) = api.py_bg_i2c_read_data_timing(beagle, max_bytes, max_timing, data_in, data_timing)
    # data_in post-processing
    if __data_in: del data_in[max(0, min(_ret_, len(data_in))):]
    # data_timing post-processing
    if __data_timing: del data_timing[max(0, min(_ret_, len(data_timing))):]
    return (_ret_, status, time_sop, time_duration, time_dataoffset, data_in, data_timing)


def bg_i2c_read_bit_timing (beagle, data_in, bit_timing):
    """usage: (int return, u32 status, u64 time_sop, u64 time_duration, u32 time_dataoffset, u16[] data_in, u32[] bit_timing) = bg_i2c_read_bit_timing(Beagle beagle, u16[] data_in, u32[] bit_timing)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function.

    Additionally, for arrays that are filled by the API function, an
    integer can be passed in place of the array argument and the API
    will automatically create an array of that length.  All output
    arrays, whether passed in or generated, are passed back in the
    returned tuple."""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # data_in pre-processing
    __data_in = isinstance(data_in, int)
    if __data_in:
        (data_in, max_bytes) = (array_u16(data_in), data_in)
    else:
        (data_in, max_bytes) = isinstance(data_in, ArrayType) and (data_in, len(data_in)) or (data_in[0], min(len(data_in[0]), int(data_in[1])))
        if data_in.typecode != 'H':
            raise TypeError("type for 'data_in' must be array('H')")
    # bit_timing pre-processing
    __bit_timing = isinstance(bit_timing, int)
    if __bit_timing:
        (bit_timing, max_timing) = (array_u32(bit_timing), bit_timing)
    else:
        (bit_timing, max_timing) = isinstance(bit_timing, ArrayType) and (bit_timing, len(bit_timing)) or (bit_timing[0], min(len(bit_timing[0]), int(bit_timing[1])))
        if bit_timing.typecode != 'I':
            raise TypeError("type for 'bit_timing' must be array('I')")
    # Call API function
    (_ret_, status, time_sop, time_duration, time_dataoffset) = api.py_bg_i2c_read_bit_timing(beagle, max_bytes, max_timing, data_in, bit_timing)
    # data_in post-processing
    if __data_in: del data_in[max(0, min(_ret_, len(data_in))):]
    # bit_timing post-processing
    if __bit_timing: del bit_timing[max(0, min(bg_bit_timing_size(BG_PROTOCOL_I2C, _ret_), len(bit_timing))):]
    return (_ret_, status, time_sop, time_duration, time_dataoffset, data_in, bit_timing)



#==========================================================================
# SPI API
#==========================================================================
# enum BeagleSpiSSPolarity
BG_SPI_SS_ACTIVE_LOW  = 0
BG_SPI_SS_ACTIVE_HIGH = 1

# enum BeagleSpiSckSamplingEdge
BG_SPI_SCK_SAMPLING_EDGE_RISING  = 0
BG_SPI_SCK_SAMPLING_EDGE_FALLING = 1

# enum BeagleSpiBitorder
BG_SPI_BITORDER_MSB = 0
BG_SPI_BITORDER_LSB = 1

def bg_spi_configure (beagle, ss_polarity, sck_sampling_edge, bitorder):
    """usage: int return = bg_spi_configure(Beagle beagle, BeagleSpiSSPolarity ss_polarity, BeagleSpiSckSamplingEdge sck_sampling_edge, BeagleSpiBitorder bitorder)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_spi_configure(beagle, ss_polarity, sck_sampling_edge, bitorder)


def bg_spi_read (beagle, data_mosi, data_miso):
    """usage: (int return, u32 status, u64 time_sop, u64 time_duration, u32 time_dataoffset, u08[] data_mosi, u08[] data_miso) = bg_spi_read(Beagle beagle, u08[] data_mosi, u08[] data_miso)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function.

    Additionally, for arrays that are filled by the API function, an
    integer can be passed in place of the array argument and the API
    will automatically create an array of that length.  All output
    arrays, whether passed in or generated, are passed back in the
    returned tuple."""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # data_mosi pre-processing
    __data_mosi = isinstance(data_mosi, int)
    if __data_mosi:
        (data_mosi, mosi_max_bytes) = (array_u08(data_mosi), data_mosi)
    else:
        (data_mosi, mosi_max_bytes) = isinstance(data_mosi, ArrayType) and (data_mosi, len(data_mosi)) or (data_mosi[0], min(len(data_mosi[0]), int(data_mosi[1])))
        if data_mosi.typecode != 'B':
            raise TypeError("type for 'data_mosi' must be array('B')")
    # data_miso pre-processing
    __data_miso = isinstance(data_miso, int)
    if __data_miso:
        (data_miso, miso_max_bytes) = (array_u08(data_miso), data_miso)
    else:
        (data_miso, miso_max_bytes) = isinstance(data_miso, ArrayType) and (data_miso, len(data_miso)) or (data_miso[0], min(len(data_miso[0]), int(data_miso[1])))
        if data_miso.typecode != 'B':
            raise TypeError("type for 'data_miso' must be array('B')")
    # Call API function
    (_ret_, status, time_sop, time_duration, time_dataoffset) = api.py_bg_spi_read(beagle, mosi_max_bytes, miso_max_bytes, data_mosi, data_miso)
    # data_mosi post-processing
    if __data_mosi: del data_mosi[max(0, min(_ret_, len(data_mosi))):]
    # data_miso post-processing
    if __data_miso: del data_miso[max(0, min(_ret_, len(data_miso))):]
    return (_ret_, status, time_sop, time_duration, time_dataoffset, data_mosi, data_miso)


def bg_spi_read_data_timing (beagle, data_mosi, data_miso, data_timing):
    """usage: (int return, u32 status, u64 time_sop, u64 time_duration, u32 time_dataoffset, u08[] data_mosi, u08[] data_miso, u32[] data_timing) = bg_spi_read_data_timing(Beagle beagle, u08[] data_mosi, u08[] data_miso, u32[] data_timing)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function.

    Additionally, for arrays that are filled by the API function, an
    integer can be passed in place of the array argument and the API
    will automatically create an array of that length.  All output
    arrays, whether passed in or generated, are passed back in the
    returned tuple."""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # data_mosi pre-processing
    __data_mosi = isinstance(data_mosi, int)
    if __data_mosi:
        (data_mosi, mosi_max_bytes) = (array_u08(data_mosi), data_mosi)
    else:
        (data_mosi, mosi_max_bytes) = isinstance(data_mosi, ArrayType) and (data_mosi, len(data_mosi)) or (data_mosi[0], min(len(data_mosi[0]), int(data_mosi[1])))
        if data_mosi.typecode != 'B':
            raise TypeError("type for 'data_mosi' must be array('B')")
    # data_miso pre-processing
    __data_miso = isinstance(data_miso, int)
    if __data_miso:
        (data_miso, miso_max_bytes) = (array_u08(data_miso), data_miso)
    else:
        (data_miso, miso_max_bytes) = isinstance(data_miso, ArrayType) and (data_miso, len(data_miso)) or (data_miso[0], min(len(data_miso[0]), int(data_miso[1])))
        if data_miso.typecode != 'B':
            raise TypeError("type for 'data_miso' must be array('B')")
    # data_timing pre-processing
    __data_timing = isinstance(data_timing, int)
    if __data_timing:
        (data_timing, max_timing) = (array_u32(data_timing), data_timing)
    else:
        (data_timing, max_timing) = isinstance(data_timing, ArrayType) and (data_timing, len(data_timing)) or (data_timing[0], min(len(data_timing[0]), int(data_timing[1])))
        if data_timing.typecode != 'I':
            raise TypeError("type for 'data_timing' must be array('I')")
    # Call API function
    (_ret_, status, time_sop, time_duration, time_dataoffset) = api.py_bg_spi_read_data_timing(beagle, mosi_max_bytes, miso_max_bytes, max_timing, data_mosi, data_miso, data_timing)
    # data_mosi post-processing
    if __data_mosi: del data_mosi[max(0, min(_ret_, len(data_mosi))):]
    # data_miso post-processing
    if __data_miso: del data_miso[max(0, min(_ret_, len(data_miso))):]
    # data_timing post-processing
    if __data_timing: del data_timing[max(0, min(_ret_, len(data_timing))):]
    return (_ret_, status, time_sop, time_duration, time_dataoffset, data_mosi, data_miso, data_timing)


def bg_spi_read_bit_timing (beagle, data_mosi, data_miso, bit_timing):
    """usage: (int return, u32 status, u64 time_sop, u64 time_duration, u32 time_dataoffset, u08[] data_mosi, u08[] data_miso, u32[] bit_timing) = bg_spi_read_bit_timing(Beagle beagle, u08[] data_mosi, u08[] data_miso, u32[] bit_timing)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function.

    Additionally, for arrays that are filled by the API function, an
    integer can be passed in place of the array argument and the API
    will automatically create an array of that length.  All output
    arrays, whether passed in or generated, are passed back in the
    returned tuple."""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # data_mosi pre-processing
    __data_mosi = isinstance(data_mosi, int)
    if __data_mosi:
        (data_mosi, mosi_max_bytes) = (array_u08(data_mosi), data_mosi)
    else:
        (data_mosi, mosi_max_bytes) = isinstance(data_mosi, ArrayType) and (data_mosi, len(data_mosi)) or (data_mosi[0], min(len(data_mosi[0]), int(data_mosi[1])))
        if data_mosi.typecode != 'B':
            raise TypeError("type for 'data_mosi' must be array('B')")
    # data_miso pre-processing
    __data_miso = isinstance(data_miso, int)
    if __data_miso:
        (data_miso, miso_max_bytes) = (array_u08(data_miso), data_miso)
    else:
        (data_miso, miso_max_bytes) = isinstance(data_miso, ArrayType) and (data_miso, len(data_miso)) or (data_miso[0], min(len(data_miso[0]), int(data_miso[1])))
        if data_miso.typecode != 'B':
            raise TypeError("type for 'data_miso' must be array('B')")
    # bit_timing pre-processing
    __bit_timing = isinstance(bit_timing, int)
    if __bit_timing:
        (bit_timing, max_timing) = (array_u32(bit_timing), bit_timing)
    else:
        (bit_timing, max_timing) = isinstance(bit_timing, ArrayType) and (bit_timing, len(bit_timing)) or (bit_timing[0], min(len(bit_timing[0]), int(bit_timing[1])))
        if bit_timing.typecode != 'I':
            raise TypeError("type for 'bit_timing' must be array('I')")
    # Call API function
    (_ret_, status, time_sop, time_duration, time_dataoffset) = api.py_bg_spi_read_bit_timing(beagle, mosi_max_bytes, miso_max_bytes, max_timing, data_mosi, data_miso, bit_timing)
    # data_mosi post-processing
    if __data_mosi: del data_mosi[max(0, min(_ret_, len(data_mosi))):]
    # data_miso post-processing
    if __data_miso: del data_miso[max(0, min(_ret_, len(data_miso))):]
    # bit_timing post-processing
    if __bit_timing: del bit_timing[max(0, min(bg_bit_timing_size(BG_PROTOCOL_SPI, _ret_), len(bit_timing))):]
    return (_ret_, status, time_sop, time_duration, time_dataoffset, data_mosi, data_miso, bit_timing)



#==========================================================================
# USB API
#==========================================================================
# USB packet PID definitions
BG_USB_PID_OUT = 0xe1
BG_USB_PID_IN = 0x69
BG_USB_PID_SOF = 0xa5
BG_USB_PID_SETUP = 0x2d
BG_USB_PID_DATA0 = 0xc3
BG_USB_PID_DATA1 = 0x4b
BG_USB_PID_DATA2 = 0x87
BG_USB_PID_MDATA = 0x0f
BG_USB_PID_ACK = 0xd2
BG_USB_PID_NAK = 0x5a
BG_USB_PID_STALL = 0x1e
BG_USB_PID_NYET = 0x96
BG_USB_PID_PRE = 0x3c
BG_USB_PID_ERR = 0x3c
BG_USB_PID_SPLIT = 0x78
BG_USB_PID_PING = 0xb4
BG_USB_PID_EXT = 0xf0
BG_USB_PID_CORRUPTED = 0xff
# The following codes are returned for USB 12 and 480 captures
BG_READ_USB_ERR_BAD_SIGNALS = 0x00010000
BG_READ_USB_ERR_BAD_PID = 0x00200000
BG_READ_USB_ERR_BAD_CRC = 0x00400000
# The following codes are only returned for USB 12 captures
BG_READ_USB_ERR_BAD_SYNC = 0x00020000
BG_READ_USB_ERR_BIT_STUFF = 0x00040000
BG_READ_USB_ERR_FALSE_EOP = 0x00080000
BG_READ_USB_ERR_LONG_EOP = 0x00100000
# The following codes are only returned for USB 480 captures
BG_READ_USB_TRUNCATION_MODE = 0x20000000
BG_READ_USB_END_OF_CAPTURE = 0x40000000
# The following events are returned for USB 12 and 480 captures
BG_EVENT_USB_HOST_DISCONNECT = 0x00000100
BG_EVENT_USB_TARGET_DISCONNECT = 0x00000200
BG_EVENT_USB_HOST_CONNECT = 0x00000400
BG_EVENT_USB_TARGET_CONNECT = 0x00000800
BG_EVENT_USB_RESET = 0x00001000
# USB 480 specific event codes
BG_EVENT_USB_CHIRP_J = 0x00002000
BG_EVENT_USB_CHIRP_K = 0x00004000
BG_EVENT_USB_SPEED_UNKNOWN = 0x00008000
BG_EVENT_USB_LOW_SPEED = 0x00010000
BG_EVENT_USB_FULL_SPEED = 0x00020000
BG_EVENT_USB_HIGH_SPEED = 0x00040000
BG_EVENT_USB_LOW_OVER_FULL_SPEED = 0x00080000
BG_EVENT_USB_SUSPEND = 0x00100000
BG_EVENT_USB_RESUME = 0x00200000
BG_EVENT_USB_KEEP_ALIVE = 0x00400000
BG_EVENT_USB_DIGITAL_INPUT = 0x00800000
BG_EVENT_USB_OTG_HNP = 0x01000000
BG_EVENT_USB_OTG_SRP_DATA_PULSE = 0x02000000
BG_EVENT_USB_OTG_SRP_VBUS_PULSE = 0x04000000
BG_EVENT_USB_DIGITAL_INPUT_MASK = 0x0000000f

#==========================================================================
# USB 12 API
#==========================================================================
def bg_usb12_read (beagle, packet):
    """usage: (int return, u32 status, u32 events, u64 time_sop, u64 time_duration, u32 time_dataoffset, u08[] packet) = bg_usb12_read(Beagle beagle, u08[] packet)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function.

    Additionally, for arrays that are filled by the API function, an
    integer can be passed in place of the array argument and the API
    will automatically create an array of that length.  All output
    arrays, whether passed in or generated, are passed back in the
    returned tuple."""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # packet pre-processing
    __packet = isinstance(packet, int)
    if __packet:
        (packet, max_bytes) = (array_u08(packet), packet)
    else:
        (packet, max_bytes) = isinstance(packet, ArrayType) and (packet, len(packet)) or (packet[0], min(len(packet[0]), int(packet[1])))
        if packet.typecode != 'B':
            raise TypeError("type for 'packet' must be array('B')")
    # Call API function
    (_ret_, status, events, time_sop, time_duration, time_dataoffset) = api.py_bg_usb12_read(beagle, max_bytes, packet)
    # packet post-processing
    if __packet: del packet[max(0, min(_ret_, len(packet))):]
    return (_ret_, status, events, time_sop, time_duration, time_dataoffset, packet)


def bg_usb12_read_data_timing (beagle, packet, data_timing):
    """usage: (int return, u32 status, u32 events, u64 time_sop, u64 time_duration, u32 time_dataoffset, u08[] packet, u32[] data_timing) = bg_usb12_read_data_timing(Beagle beagle, u08[] packet, u32[] data_timing)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function.

    Additionally, for arrays that are filled by the API function, an
    integer can be passed in place of the array argument and the API
    will automatically create an array of that length.  All output
    arrays, whether passed in or generated, are passed back in the
    returned tuple."""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # packet pre-processing
    __packet = isinstance(packet, int)
    if __packet:
        (packet, max_bytes) = (array_u08(packet), packet)
    else:
        (packet, max_bytes) = isinstance(packet, ArrayType) and (packet, len(packet)) or (packet[0], min(len(packet[0]), int(packet[1])))
        if packet.typecode != 'B':
            raise TypeError("type for 'packet' must be array('B')")
    # data_timing pre-processing
    __data_timing = isinstance(data_timing, int)
    if __data_timing:
        (data_timing, max_timing) = (array_u32(data_timing), data_timing)
    else:
        (data_timing, max_timing) = isinstance(data_timing, ArrayType) and (data_timing, len(data_timing)) or (data_timing[0], min(len(data_timing[0]), int(data_timing[1])))
        if data_timing.typecode != 'I':
            raise TypeError("type for 'data_timing' must be array('I')")
    # Call API function
    (_ret_, status, events, time_sop, time_duration, time_dataoffset) = api.py_bg_usb12_read_data_timing(beagle, max_bytes, max_timing, packet, data_timing)
    # packet post-processing
    if __packet: del packet[max(0, min(_ret_, len(packet))):]
    # data_timing post-processing
    if __data_timing: del data_timing[max(0, min(_ret_, len(data_timing))):]
    return (_ret_, status, events, time_sop, time_duration, time_dataoffset, packet, data_timing)


def bg_usb12_read_bit_timing (beagle, packet, bit_timing):
    """usage: (int return, u32 status, u32 events, u64 time_sop, u64 time_duration, u32 time_dataoffset, u08[] packet, u32[] bit_timing) = bg_usb12_read_bit_timing(Beagle beagle, u08[] packet, u32[] bit_timing)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function.

    Additionally, for arrays that are filled by the API function, an
    integer can be passed in place of the array argument and the API
    will automatically create an array of that length.  All output
    arrays, whether passed in or generated, are passed back in the
    returned tuple."""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # packet pre-processing
    __packet = isinstance(packet, int)
    if __packet:
        (packet, max_bytes) = (array_u08(packet), packet)
    else:
        (packet, max_bytes) = isinstance(packet, ArrayType) and (packet, len(packet)) or (packet[0], min(len(packet[0]), int(packet[1])))
        if packet.typecode != 'B':
            raise TypeError("type for 'packet' must be array('B')")
    # bit_timing pre-processing
    __bit_timing = isinstance(bit_timing, int)
    if __bit_timing:
        (bit_timing, max_timing) = (array_u32(bit_timing), bit_timing)
    else:
        (bit_timing, max_timing) = isinstance(bit_timing, ArrayType) and (bit_timing, len(bit_timing)) or (bit_timing[0], min(len(bit_timing[0]), int(bit_timing[1])))
        if bit_timing.typecode != 'I':
            raise TypeError("type for 'bit_timing' must be array('I')")
    # Call API function
    (_ret_, status, events, time_sop, time_duration, time_dataoffset) = api.py_bg_usb12_read_bit_timing(beagle, max_bytes, max_timing, packet, bit_timing)
    # packet post-processing
    if __packet: del packet[max(0, min(_ret_, len(packet))):]
    # bit_timing post-processing
    if __bit_timing: del bit_timing[max(0, min(bg_bit_timing_size(BG_PROTOCOL_USB, _ret_), len(bit_timing))):]
    return (_ret_, status, events, time_sop, time_duration, time_dataoffset, packet, bit_timing)



#==========================================================================
# USB 480 API
#==========================================================================
# General constants
BG_USB480_TRUNCATION_LENGTH = 4
# Capture modes
# enum BeagleUsb480CaptureMode
BG_USB480_CAPTURE_REALTIME                 = 0
BG_USB480_CAPTURE_REALTIME_WITH_PROTECTION = 1
BG_USB480_CAPTURE_DELAYED_DOWNLOAD         = 2

# Target speeds
# enum BeagleUsb480TargetSpeed
BG_USB480_AUTO_SPEED_DETECT = 0
BG_USB480_LOW_SPEED         = 1
BG_USB480_FULL_SPEED        = 2
BG_USB480_HIGH_SPEED        = 3

def bg_usb480_capture_configure (beagle, capture_mode, target_speed):
    """usage: int return = bg_usb480_capture_configure(Beagle beagle, BeagleUsb480CaptureMode capture_mode, BeagleUsb480TargetSpeed target_speed)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_usb480_capture_configure(beagle, capture_mode, target_speed)


# Digital output configuration
BG_USB480_DIGITAL_OUT_ENABLE_PIN1 = 0x01
BG_USB480_DIGITAL_OUT_PIN1_ACTIVE_HIGH = 0x01
BG_USB480_DIGITAL_OUT_PIN1_ACTIVE_LOW = 0x00
BG_USB480_DIGITAL_OUT_ENABLE_PIN2 = 0x02
BG_USB480_DIGITAL_OUT_PIN2_ACTIVE_HIGH = 0x02
BG_USB480_DIGITAL_OUT_PIN2_ACTIVE_LOW = 0x00
BG_USB480_DIGITAL_OUT_ENABLE_PIN3 = 0x04
BG_USB480_DIGITAL_OUT_PIN3_ACTIVE_HIGH = 0x04
BG_USB480_DIGITAL_OUT_PIN3_ACTIVE_LOW = 0x00
BG_USB480_DIGITAL_OUT_ENABLE_PIN4 = 0x08
BG_USB480_DIGITAL_OUT_PIN4_ACTIVE_HIGH = 0x08
BG_USB480_DIGITAL_OUT_PIN4_ACTIVE_LOW = 0x00
def bg_usb480_digital_out_config (beagle, out_enable_mask, out_polarity_mask):
    """usage: int return = bg_usb480_digital_out_config(Beagle beagle, u08 out_enable_mask, u08 out_polarity_mask)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_usb480_digital_out_config(beagle, out_enable_mask, out_polarity_mask)


# Digital output match pin configuration
# enum BeagleUsb480DigitalOutMatchPins
BG_USB480_DIGITAL_OUT_MATCH_PIN3 = 3
BG_USB480_DIGITAL_OUT_MATCH_PIN4 = 4

# Packet matching modes
# enum BeagleUsb480MatchType
BG_USB480_MATCH_TYPE_DISABLED  = 0
BG_USB480_MATCH_TYPE_EQUAL     = 1
BG_USB480_MATCH_TYPE_NOT_EQUAL = 2

# Digital ouput matching configuration
class BeagleUsb480PacketMatch:
    def __init__ (self):
        self.pid_match_type = 0
        self.pid_match_val  = 0
        self.dev_match_type = 0
        self.dev_match_val  = 0
        self.ep_match_type  = 0
        self.ep_match_val   = 0

# Data match PID mask
BG_USB480_DATA_MATCH_DATA0 = 0x01
BG_USB480_DATA_MATCH_DATA1 = 0x02
BG_USB480_DATA_MATCH_DATA2 = 0x04
BG_USB480_DATA_MATCH_MDATA = 0x08
class BeagleUsb480DataMatch:
    def __init__ (self):
        self.data_match_type   = 0
        self.data_match_pid    = 0
        self.data              = array('B')
        self.data_valid        = array('B')

def bg_usb480_digital_out_match (beagle, pin_num, packet_match, data_match):
    """usage: int return = bg_usb480_digital_out_match(Beagle beagle, BeagleUsb480DigitalOutMatchPins pin_num, BeagleUsb480PacketMatch packet_match, BeagleUsb480DataMatch data_match)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # packet_match pre-processing
    c_packet_match = None
    if packet_match != None:
        c_packet_match = (packet_match.pid_match_type, packet_match.pid_match_val, packet_match.dev_match_type, packet_match.dev_match_val, packet_match.ep_match_type, packet_match.ep_match_val)
    # data_match pre-processing
    c_data_match = None
    if data_match != None:
        c_data_match = (data_match.data_match_type, data_match.data_match_pid, data_match.data.buffer_info(), data_match.data_valid.buffer_info())
    # Call API function
    return api.py_bg_usb480_digital_out_match(beagle, pin_num, c_packet_match, c_data_match)


# Digital input pin configuration
BG_USB480_DIGITAL_IN_ENABLE_PIN1 = 0x01
BG_USB480_DIGITAL_IN_ENABLE_PIN2 = 0x02
BG_USB480_DIGITAL_IN_ENABLE_PIN3 = 0x04
BG_USB480_DIGITAL_IN_ENABLE_PIN4 = 0x08
def bg_usb480_digital_in_config (beagle, in_enable_mask):
    """usage: int return = bg_usb480_digital_in_config(Beagle beagle, u08 in_enable_mask)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_usb480_digital_in_config(beagle, in_enable_mask)


# Hardware filtering configuration
BG_USB480_HW_FILTER_PID_SOF = 0x01
BG_USB480_HW_FILTER_PID_IN = 0x02
BG_USB480_HW_FILTER_PID_PING = 0x04
BG_USB480_HW_FILTER_PID_PRE = 0x08
BG_USB480_HW_FILTER_PID_SPLIT = 0x10
BG_USB480_HW_FILTER_SELF = 0x20
def bg_usb480_hw_filter_config (beagle, filter_enable_mask):
    """usage: int return = bg_usb480_hw_filter_config(Beagle beagle, u08 filter_enable_mask)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_usb480_hw_filter_config(beagle, filter_enable_mask)


def bg_usb480_hw_buffer_stats (beagle):
    """usage: (int return, u32 buffer_size, u32 buffer_usage, u08 buffer_full) = bg_usb480_hw_buffer_stats(Beagle beagle)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_usb480_hw_buffer_stats(beagle)


def bg_usb480_read (beagle, packet):
    """usage: (int return, u32 status, u32 events, u64 time_sop, u64 time_duration, u32 time_dataoffset, u08[] packet) = bg_usb480_read(Beagle beagle, u08[] packet)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function.

    Additionally, for arrays that are filled by the API function, an
    integer can be passed in place of the array argument and the API
    will automatically create an array of that length.  All output
    arrays, whether passed in or generated, are passed back in the
    returned tuple."""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # packet pre-processing
    __packet = isinstance(packet, int)
    if __packet:
        (packet, max_bytes) = (array_u08(packet), packet)
    else:
        (packet, max_bytes) = isinstance(packet, ArrayType) and (packet, len(packet)) or (packet[0], min(len(packet[0]), int(packet[1])))
        if packet.typecode != 'B':
            raise TypeError("type for 'packet' must be array('B')")
    # Call API function
    (_ret_, status, events, time_sop, time_duration, time_dataoffset) = api.py_bg_usb480_read(beagle, max_bytes, packet)
    # packet post-processing
    if __packet: del packet[max(0, min(_ret_, len(packet))):]
    return (_ret_, status, events, time_sop, time_duration, time_dataoffset, packet)


def bg_usb480_reconstruct_timing (speed, packet, bit_timing):
    """usage: (int return, u32[] bit_timing) = bg_usb480_reconstruct_timing(BeagleUsb480TargetSpeed speed, u08[] packet, u32[] bit_timing)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function.

    Additionally, for arrays that are filled by the API function, an
    integer can be passed in place of the array argument and the API
    will automatically create an array of that length.  All output
    arrays, whether passed in or generated, are passed back in the
    returned tuple."""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # packet pre-processing
    (packet, num_bytes) = isinstance(packet, ArrayType) and (packet, len(packet)) or (packet[0], min(len(packet[0]), int(packet[1])))
    if packet.typecode != 'B':
        raise TypeError("type for 'packet' must be array('B')")
    # bit_timing pre-processing
    __bit_timing = isinstance(bit_timing, int)
    if __bit_timing:
        (bit_timing, max_timing) = (array_u32(bit_timing), bit_timing)
    else:
        (bit_timing, max_timing) = isinstance(bit_timing, ArrayType) and (bit_timing, len(bit_timing)) or (bit_timing[0], min(len(bit_timing[0]), int(bit_timing[1])))
        if bit_timing.typecode != 'I':
            raise TypeError("type for 'bit_timing' must be array('I')")
    # Call API function
    (_ret_) = api.py_bg_usb480_reconstruct_timing(speed, num_bytes, packet, max_timing, bit_timing)
    # bit_timing post-processing
    if __bit_timing: del bit_timing[max(0, min(max_timing, len(bit_timing))):]
    return (_ret_, bit_timing)



#==========================================================================
# MDIO API
#==========================================================================
# enum BeagleMdioClause
BG_MDIO_CLAUSE_22    = 0
BG_MDIO_CLAUSE_45    = 1
BG_MDIO_CLAUSE_ERROR = 2

BG_MDIO_OPCODE22_WRITE = 0x01
BG_MDIO_OPCODE22_READ = 0x02
BG_MDIO_OPCODE22_ERROR = 0xff
BG_MDIO_OPCODE45_ADDR = 0x00
BG_MDIO_OPCODE45_WRITE = 0x01
BG_MDIO_OPCODE45_READ_POSTINC = 0x02
BG_MDIO_OPCODE45_READ = 0x03
# Read the next MDIO frame.
def bg_mdio_read (beagle):
    """usage: (int return, u32 status, u64 time_sop, u64 time_duration, u32 time_dataoffset, u32 data_in) = bg_mdio_read(Beagle beagle)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_mdio_read(beagle)


def bg_mdio_read_bit_timing (beagle, bit_timing):
    """usage: (int return, u32 status, u64 time_sop, u64 time_duration, u32 time_dataoffset, u32 data_in, u32[] bit_timing) = bg_mdio_read_bit_timing(Beagle beagle, u32[] bit_timing)

    All arrays can be passed into the API as an ArrayType object or as
    a tuple (array, length), where array is an ArrayType object and
    length is an integer.  The user-specified length would then serve
    as the length argument to the API funtion (please refer to the
    product datasheet).  If only the array is provided, the array's
    intrinsic length is used as the argument to the underlying API
    function.

    Additionally, for arrays that are filled by the API function, an
    integer can be passed in place of the array argument and the API
    will automatically create an array of that length.  All output
    arrays, whether passed in or generated, are passed back in the
    returned tuple."""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # bit_timing pre-processing
    __bit_timing = isinstance(bit_timing, int)
    if __bit_timing:
        (bit_timing, max_timing) = (array_u32(bit_timing), bit_timing)
    else:
        (bit_timing, max_timing) = isinstance(bit_timing, ArrayType) and (bit_timing, len(bit_timing)) or (bit_timing[0], min(len(bit_timing[0]), int(bit_timing[1])))
        if bit_timing.typecode != 'I':
            raise TypeError("type for 'bit_timing' must be array('I')")
    # Call API function
    (_ret_, status, time_sop, time_duration, time_dataoffset, data_in) = api.py_bg_mdio_read_bit_timing(beagle, max_timing, bit_timing)
    # bit_timing post-processing
    if __bit_timing: del bit_timing[max(0, min(bg_bit_timing_size(BG_PROTOCOL_MDIO, _ret_), len(bit_timing))):]
    return (_ret_, status, time_sop, time_duration, time_dataoffset, data_in, bit_timing)


# Parse the raw MDIO data into the standard format.
# This function will fill the supplied fields as per
# the constants defined above.  If the raw data contains
# a malformed turnaround field, the caller will be
# notified of the error through the return value of
# this function (BG_MDIO_BAD_TURNAROUND).
def bg_mdio_parse (packet):
    """usage: (int return, u08 clause, u08 opcode, u08 addr1, u08 addr2, u16 data) = bg_mdio_parse(u32 packet)"""

    if not BG_LIBRARY_LOADED: return BG_INCOMPATIBLE_LIBRARY
    # Call API function
    return api.py_bg_mdio_parse(packet)


